package com.icontrols.kakao.common;

import java.util.Arrays;

import org.slf4j.LoggerFactory;

import com.icontrols.kakao.common.utils.BeanUtils;
import com.icontrols.kakao.repo.ThingsInfoRepository;
import com.icontrols.kakao.socket.wallpad.KakaoMsgHandler;

import ch.qos.logback.classic.Logger;

public final class THINGS_INFO {
	
	public static final int THINGS_LIGHT 		= 1,  THINGS_TEMPER  	= 2,  THINGS_ELECPLUG 	= 3,  THINGS_GAS 		= 4,
							THINGS_ILGALSWITCH 	= 5,  THINGS_WALLPAD  	= 6,  THINGS_VENTIL 	= 7,  THINGS_NOTICE 	= 8,
							THINGS_VOTE			= 9,  THINGS_PARKING  	= 10, THINGS_PARCEL 	= 11, THINGS_HEMS 		= 12,
							THINGS_CARIN 		= 13, THINGS_ELEVATOR 	= 14, THINGS_VISITOR 	= 15, THINGS_EMERGENCY 	= 16,
							THINGS_LIGHT_DIMMER = 17, THINGS_COLOR_LIGHT_DIMMER = 18, THINGS_AIR_MONITOR = 19, THINGS_AIRCON = 20;
	
	public enum THINGS_CAPABILITY{
		LIVINGLIGHT("livingLight","{\"capabilities\" : [ { \"name\": \"Power\", \"attributes\": { \"power\": { \"writable\": [ \"on\", \"off\" ], \"supported\": [ \"on\", \"off\" ], \"mutability\": \"rw\" } } } ]}"),
		LIGHT("light","{\"capabilities\" : [ { \"name\": \"Power\", \"attributes\": { \"power\": { \"writable\": [ \"on\", \"off\" ], \"supported\": [ \"on\", \"off\" ], \"mutability\": \"rw\" } } } ]}"),
		LIGHT_DIMMER("dimming","{\"capabilities\" : [ { \"name\": \"Power\", \"attributes\": { \"power\": { \"writable\": [ \"on\", \"off\" ], \"supported\": [ \"on\", \"off\" ], \"mutability\": \"rw\" } } }, { \"name\": \"Brightness\", \"attributes\": { \"brightness\": { \"max\": 10, \"min\": 1, \"mutability\": \"rw\" } } } ]}"),
		COLOR_LIGHT("colorLight", "{\"capabilities\" : [ { \"name\": \"Power\", \"attributes\": { \"power\": { \"writable\": [ \"on\", \"off\" ], \"supported\": [ \"on\", \"off\" ], \"mutability\": \"rw\" } } }, { \"name\": \"ColorMode\", \"attributes\": { \"colorMode\": { \"writable\": [ \"colorTemperature\" ], \"supported\": [ \"colorTemperature\" ], \"mutability\": \"rw\" } } }, { \"name\": \"ColorTemperature\", \"attributes\": { \"colorTemperature\": { \"max\": 5700, \"min\": 3000, \"mutability\": \"rw\" } } } ]}"),
		COLOR_LIGHT_DIMMER("colorDimming", "{\"capabilities\" : [ { \"name\": \"Power\", \"attributes\": { \"power\": { \"writable\": [ \"on\", \"off\" ], \"supported\": [ \"on\", \"off\" ], \"mutability\": \"rw\" } } }, { \"name\": \"Brightness\", \"attributes\": { \"brightness\": { \"max\": 10, \"min\": 1, \"mutability\": \"rw\" } } }, { \"name\": \"ColorMode\", \"attributes\": { \"colorMode\": { \"writable\": [ \"colorTemperature\" ], \"supported\": [ \"colorTemperature\" ], \"mutability\": \"rw\" } } }, { \"name\": \"ColorTemperature\", \"attributes\": { \"colorTemperature\": { \"max\": 5700, \"min\": 3000, \"mutability\": \"rw\" } } } ]}"),
		TEMPER("temper", "{\"capabilities\" : [ { \"name\": \"Power\", \"attributes\": { \"power\": { \"writable\": [ \"on\", \"off\" ], \"supported\": [ \"on\", \"off\" ], \"mutability\": \"rw\" } } }, { \"name\": \"Temperature\", \"attributes\": { \"currentTemperature\": { \"mutability\": \"r\", \"max\": 40, \"min\": 8 } } }, { \"name\": \"TemperatureSetpoint\", \"attributes\": { \"targetSetpoint\": { \"min\": 18, \"max\": 30, \"mutability\": \"rw\" } } } ]}"),
		VENTIL("ventil","{\"capabilities\" : [ { \"name\": \"Power\", \"attributes\": { \"power\": { \"writable\": [ \"on\", \"off\" ], \"supported\": [ \"on\", \"off\" ], \"mutability\": \"rw\" } } }, { \"name\": \"AirFlow\", \"attributes\": { \"fanSpeed\": { \"writable\": [ \"low\", \"middle\", \"high\" ], \"supported\": [ \"low\", \"middle\", \"high\" ], \"mutability\": \"rw\" } } } ]}"),
		GAS("gas","{\"capabilities\" : [ { \"name\" : \"Lock\" } ] }"),
		ELEVATOR("elevator", "{ \"capabilities\" : [ { \"name\":\"ElevatorCall\", \"attributes\":{ \"callDirection\":{ \"mutability\":\"rw\", \"writable\":[\"down\"], \"supported\":[\"down\"] } } } ] }"),
		ELECTRIC("electric", "{\"capabilities\" : [ { \"name\": \"Power\", \"attributes\": { \"power\": { \"writable\": [ \"on\", \"off\" ], \"supported\": [ \"on\", \"off\" ], \"mutability\": \"rw\" } } } ]}"),
		PARKING("parcel", "{\"capabilities\" : [ { \"name\" : \"Parking\" , \"attributes\" : { \"parkingLocations\": { \"mutability\":\"r\" } } } ]}"),
		AIRSENSOR("airsensor", "{ \"capabilities\": [ { \"name\": \"AirQuality\", \"attributes\": { \"co2\": { \"max\": 65534, \"min\": 0, \"mutability\": \"r\" }, \"pm10\": { \"max\": 65534, \"min\": 0, \"mutability\": \"r\" }, \"pm2_5\": { \"max\": 65534, \"min\": 0, \"mutability\": \"r\" }, \"qualityIndex\": { \"supported\": [ \"good\", \"moderate\", \"unhealthy\", \"very_unhealthy\" ], \"mutability\": \"r\" } } } ]}"),
		PARCEL("parcel", "{\"capabilities\": [ { \"name\": \"Parcel\", \"attributes\": { \"parcelLocations\": { \"mutability\": \"r\" } } } ] }"),
		AIRCON("aircon", "{ \"capabilities\": [ { \"name\": \"Power\", \"attributes\": { \"power\": { \"writable\": [ \"on\", \"off\" ], \"supported\": [ \"on\", \"off\" ], \"mutability\": \"rw\" } } }, { \"name\": \"AirConditioner\", \"attributes\": { \"mode\": { \"writable\": [ \"cool\", \"dehumidify\", \"fan_only\" ], \"supported\": [ \"cool\", \"dehumidify\", \"fan_only\" ], \"mutability\": \"rw\" } } }, { \"name\": \"AirFlow\", \"attributes\": { \"fanSpeed\": { \"writable\": [ \"low\", \"middle\", \"high\" ], \"supported\": [ \"low\", \"middle\", \"high\" ], \"mutability\": \"rw\" } } }, { \"name\": \"Temperature\", \"attributes\": { \"currentTemperature\": { \"mutability\": \"r\", \"max\": 30, \"min\": 18 } } }, { \"name\": \"TemperatureSetpoint\", \"attributes\": { \"targetSetpoint\": { \"min\": 18, \"max\": 30, \"mutability\": \"rw\" } } } ] }");
		
		private String thingsName;
		private String capJsonStr;
		
		private THINGS_CAPABILITY(String thingsName, String capJsonStr) {
			this.thingsName = thingsName;
			this.capJsonStr = capJsonStr;
		}
		public String getThingsName() {
			return thingsName;
		}
		public void setThingsName(String thingsName) {
			this.thingsName = thingsName;
		}
		public String getCapJsonStr() {
			return capJsonStr;
		}
		public void setCapJsonStr(String capJsonStr) {
			this.capJsonStr = capJsonStr;
		}
	}
	
	public enum DEVICE_DETAIL{
		LIVINGLIGHT					("livinglight", 				"Light", 			"거실조명", 	"거실"),
		LIGHT01						("light01",						"Light",			"주방조명", 	"주방"),
		LIGHT02						("light02",						"Light",			"안방조명", 	"안방"),
		LIGHT03						("light03",						"Light",			"방1조명", 	"방1"),
		LIGHT04						("light04",						"Light",			"방2조명", 	"방2"),
		LIGHT05						("light05",						"Light",			"방3조명", 	"방3"),
		LIGHT06						("light06",						"Light",			"방4조명", 	"방4"),
		LIGHT07						("light07",						"Light",			"방5조명", 	"방5"),
		LIGHT08						("light08",						"Light",			"방6조명", 	"방6"),
		LIGHT09						("light09",						"Light",			"방7조명", 	"방7"),
		LIGHT10						("light10",						"Light",			"방8조명", 	"방8"),
		DIMMER_LIVINGLIGHT			("dimmer_livinglight", 			"LightDimmer", 		"거실조명", 	"거실"),
		DIMMER_LIGHT01				("dimmer_light01",				"LightDimmer",		"주방조명", 	"주방"),
		DIMMER_LIGHT02				("dimmer_light02",				"LightDimmer",		"안방조명", 	"안방"),
		DIMMER_LIGHT03				("dimmer_light03",				"LightDimmer",		"방1조명", 	"방1"),
		DIMMER_LIGHT04				("dimmer_light04",				"LightDimmer",		"방2조명", 	"방2"),
		DIMMER_LIGHT05				("dimmer_light05",				"LightDimmer",		"방3조명", 	"방3"),
		DIMMER_LIGHT06				("dimmer_light06",				"LightDimmer",		"방4조명", 	"방4"),
		DIMMER_LIGHT07				("dimmer_light07",				"LightDimmer",		"방5조명", 	"방5"),
		DIMMER_LIGHT08				("dimmer_light08",				"LightDimmer",		"방6조명", 	"방6"),
		DIMMER_LIGHT09				("dimmer_light09",				"LightDimmer",		"방7조명", 	"방7"),
		DIMMER_LIGHT10				("dimmer_light10",				"LightDimmer",		"방8조명", 	"방8"),
		COLOR_DIMMER_LIVINGLIGHT	("colorDimmer_livinglight", 	"ColorLightDimmer",	"거실조명", 	"거실"),
		COLOR_DIMMER_LIGHT01		("colorDimmer_light01",			"ColorLightDimmer",	"주방조명", 	"주방"),
		COLOR_DIMMER_LIGHT02		("colorDimmer_light02",			"ColorLightDimmer",	"안방조명", 	"안방"),
		COLOR_DIMMER_LIGHT03		("colorDimmer_light03",			"ColorLightDimmer",	"방1조명", 	"방1"),
		COLOR_DIMMER_LIGHT04		("colorDimmer_light04",			"ColorLightDimmer",	"방2조명", 	"방2"),
		COLOR_DIMMER_LIGHT05		("colorDimmer_light05",			"ColorLightDimmer",	"방3조명", 	"방3"),
		COLOR_DIMMER_LIGHT06		("colorDimmer_light06",			"ColorLightDimmer",	"방4조명", 	"방4"),
		COLOR_DIMMER_LIGHT07		("colorDimmer_light07",			"ColorLightDimmer",	"방5조명", 	"방5"),
		COLOR_DIMMER_LIGHT08		("colorDimmer_light08",			"ColorLightDimmer",	"방6조명", 	"방6"),
		COLOR_DIMMER_LIGHT09		("colorDimmer_light09",			"ColorLightDimmer",	"방7조명", 	"방7"),
		COLOR_DIMMER_LIGHT10		("colorDimmer_light10",			"ColorLightDimmer",	"방8조명", 	"방8"),
		VENTIL						("ventil",						"AirVentilator",	"환기", 		"거실"),
		GAS							("gas",							"GasValve",			"가스밸브", 	"주방"),
		ELEVATOR					("elevator",					"ElevatorCaller",	"엘리베이터",	"공용부"),
		TEMPER1						("temper_1",					"FloorHeater",		"거실난방",	"거실"),
		TEMPER2						("temper_2",					"FloorHeater",		"안방난방",	"안방"),
		TEMPER3						("temper_3",					"FloorHeater",		"방1난방",	"방1"),
		TEMPER4						("temper_4",					"FloorHeater",		"방2난방",	"방2"),
		TEMPER5						("temper_5",					"FloorHeater",		"방3난방",	"방3"),
		TEMPER6						("temper_6",					"FloorHeater",		"방4난방",	"방4"),
		TEMPER7						("temper_7",					"FloorHeater",		"방5난방",	"방5"),
		TEMPER8						("temper_8",					"FloorHeater",		"방6난방", 	"방6"),
		TEMPER9						("temper_9",					"FloorHeater",		"방7난방", 	"방7"),
		TEMPER10					("temper_10",					"FloorHeater",		"방8난방", 	"방8"),
		ELECTRIC01					("electric01",					"PowerOutlet",		"거실콘센트", 	"거실"),
		ELECTRIC02					("electric02",					"PowerOutlet",		"주방콘센트", 	"주방"),
		ELECTRIC03					("electric03",					"PowerOutlet",		"안방콘센트", 	"안방"),
		ELECTRIC04					("electric04",					"PowerOutlet",		"방1콘센트", 	"방1"),
		ELECTRIC05					("electric05",					"PowerOutlet",		"방2콘센트", 	"방2"),
		ELECTRIC06					("electric06",					"PowerOutlet",		"방3콘센트", 	"방3"),
		ELECTRIC07					("electric07",					"PowerOutlet",		"방4콘센트", 	"방4"),
		ELECTRIC08					("electric08",					"PowerOutlet",		"방5콘센트", 	"방5"),
		ELECTRIC09					("electric09",					"PowerOutlet",		"방6콘센트", 	"방6"),
		ELECTRIC10					("electric10",					"PowerOutlet",		"방7콘센트", 	"방7"),
		ELECTRIC11					("electric11",					"PowerOutlet",		"방8콘센트", 	"방8"),
		DOORLOCK					("doorlock",					"DoorLock",			"도어락", 	"공용부"),
		PARKING						("parking",						"CommonParking",	"주차위치", 	"공용부"),
		AIRSENSOR					("airsensor",					"AirMonitor",		"공기질측정기",	"거실"),
		PARCEL						("parcel",						"CommonParcel",		"택배조회", 	"공용부"),
		AIRCON1						("aircon_1",					"AirConditioner",	"거실에어컨",	"거실"),
		AIRCON2						("aircon_2",					"AirConditioner",	"안방에어컨",	"안방"),
		AIRCON3						("aircon_3",					"AirConditioner",	"방1에어컨",	"방1"),
		AIRCON4						("aircon_4",					"AirConditioner",	"방2에어컨",	"방2"),
		AIRCON5						("aircon_5",					"AirConditioner",	"방3에어컨",	"방3"),
		AIRCON6						("aircon_6",					"AirConditioner",	"방4에어컨",	"방4"),
		AIRCON7						("aircon_7",					"AirConditioner",	"방5에어컨",	"방5"),
		AIRCON8						("aircon_8",					"AirConditioner",	"방6에어컨", 	"방6"),
		AIRCON9						("aircon_9",					"AirConditioner",	"방7에어컨", 	"방7"),
		AIRCON10					("aircon_10",					"AirConditioner",	"방8에어컨", 	"방8")
		;
		
		
		private String modelId;
		
		private String displayType;
		
		private String displayName;

		private String roomName;
		
		private DEVICE_DETAIL(String modelId, String displayType, String displayName, String roomName) {
			this.modelId = modelId;
			this.displayType = displayType;
			this.displayName = displayName;
			this.roomName = roomName;
		}

		public String getModelId() {
			return modelId;
		}

		public void setModelId(String modelId) {
			this.modelId = modelId;
		}

		public String getDisplayType() {
			return displayType;
		}

		public void setDisplayType(String displayType) {
			this.displayType = displayType;
		}

		public String getDisplayName() {
			return displayName;
		}

		public void setDisplayName(String displayName) {
			this.displayName = displayName;
		}
		
		public String getRoomName() {
			return roomName;
		}

		public void setRoomName(String roomName) {
			this.roomName = roomName;
		}
		
	}
	
	public enum THINGS{
		//endpoint는 카카오 API와 연동될 기기 endpoint
		// name 은 우리쪽에서 정의된 기기 명
		// desc 는 기기 설명
		LIGHT			(1,	"LIGHT", 			"Light", 				"조명스위치"),				
		TEMPER			(2, "TEMPER", 			"FloorHeater", 			"난방"),
		ELECPLUG		(3, "ELECTRIC", 		"PowerOutlet", 			"전기"),
		GAS				(4, "GAS", 				"GasValve", 			"가스밸브"),
		ILGALSWITCH		(5, "ILGALSWITCH", 		"ilgal", 				"일괄조명스위치"),
		WALLPAD			(6, "WALLPAD", 			"walpad", 				"월패드"),
		VENTIL			(7, "VENTIL", 			"AirVentilator", 		"환기시스템"),
		NOTICE			(8, "NOTICE", 			"InformationReporter", 	"공지사항"),
		VOTE			(9, "VOTE", 			"vote", 				"주민투표"),
		PARKING			(10, "PARKING", 		"CommonParking", 		"주차위치"),
		PARCEL			(11, "PARCEL", 			"CommonParcel", 		"택배"),
		HEMS			(12, "HEMS", 			"hems", 				"원격검침"),
		CARIN			(13, "CARIN", 			"carin", 				"주차관제(입차/출차)"),
		ELEVATOR		(14, "elevator",		"ElevatorCaller", 		"엘리베이터"),
		VISITOR			(15, "VISITOR", 		"visitor", 				"방문자"),
		EMERGENCY		(16, "EMERGENCY", 		"emergency", 			"방범"),
		LIGHTDIMMER		(17, "DIMMER", 			"LightDimmer", 			"디밍제어 조명"),
		COLORLIGHTDIMMER(18, "COLORDIMMER", 	"ColorLightDimmer", 	"색온도 디밍제어 조명"),
		AIRSENSOR		(19, "AIRSENSOR",	 	"AirMonitor", 			"공기 질 측정기"),
		AIRCON			(20, "AIRCON",	 		"AirConditioner", 		"에어컨")
		
		;
		
		private int endPoint;
		private String thingsName;
		private String kakaoDisplayName;
		private String desc;
		
		private THINGS(int endPoint, String thingsName, String kakaoDisplayName, String desc) {
			this.endPoint = endPoint;
			this.thingsName = thingsName;
			this.kakaoDisplayName = kakaoDisplayName;
			this.desc = desc;
		}

		public int getEndPoint() {
			return endPoint;
		}

		public void setEndPoint(int endPoint) {
			this.endPoint = endPoint;
		}

		public String getThingsName() {
			return thingsName;
		}

		public void setThingsName(String thingsName) {
			this.thingsName = thingsName;
		}

		public String getKakaoDisplayName() {
			return kakaoDisplayName;
		}

		public void setKakaoDisplayName(String kakaoDisplayName) {
			this.kakaoDisplayName = kakaoDisplayName;
		}

		public String getDesc() {
			return desc;
		}

		public void setDesc(String desc) {
			this.desc = desc;
		}
	}
	
	public static boolean isExistThings(String modelId) {
		THINGS_INFO.THINGS things = getThingsType(modelId);
		
		return things == null ? false:true;
	}     
	
	public static THINGS_INFO.DEVICE_DETAIL getDeviceInfo(String modelId, String status){
		THINGS_INFO.DEVICE_DETAIL[] list = THINGS_INFO.DEVICE_DETAIL.values();
		
		for(THINGS_INFO.DEVICE_DETAIL deviceDetail : list) {
			if(deviceDetail.getModelId().equals(modelId)) {
				return deviceDetail;
			}
		}
		return null; 
	}
	
	public static THINGS_INFO.THINGS getThingsType(String modelId){    
		ThingsInfoRepository repo = (ThingsInfoRepository)BeanUtils.getBean("thingsInfoRepository");
		
		THINGS_INFO.THINGS[] list = THINGS_INFO.THINGS.values();
		
		String[] splitModelId = modelId.split("_");
		
		for(THINGS_INFO.THINGS things:list) {
//			if(modelId.toLowerCase().contains(things.getThingsName().toLowerCase())) {
//				return things;
//			}
			if(splitModelId[0].equalsIgnoreCase(things.getThingsName()) || splitModelId[0].contains(things.getThingsName().toLowerCase())) {
				System.out.println(splitModelId[0] + "--" + things.getThingsName());
				return things;
			}
		}
		System.out.println("[ERROR] - " + modelId + " --> NULL");
		return null;
	}
	
	public static boolean isAvailableThings(String modelId) {
		return Arrays.asList("aircon", "airsensor", "electric", "gas", "light", "livinglight", "temper", "ventil").contains(modelId);
		
	}
	
}
