package com.icontrols.kakao.common.utils;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Base64;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.SERVERINFO;

public class CommonUtil {
	private static final Logger logger = LoggerFactory.getLogger(CommonUtil.class);
	
	public static boolean isWindows() {
		String OS = System.getProperty("os.name").toLowerCase();

		if (OS.indexOf("win") >= 0) {
			return true;
		}
		return false;
	}
	
	public static boolean getDevServer() {
		File checkFile = new File(SERVERINFO.DEV_SERVER);

		if (checkFile.exists()) {
			SERVERINFO.SendEventURL = SERVERINFO.SendDevEventURL;
			SERVERINFO.SendEventPort = SERVERINFO.SendDevEventPort;
			return true;
		} else
			return false;
	}
	
	
	public static boolean getUpdateNoticeNewDanjiServer() {
		File checkFile = new File(SERVERINFO.UPDATE_NOTICE_KIND);

		if (checkFile.exists())
			return true;
		else
			return false;
	}
	
	/**
	 * Gas Only One은 왜 그런걸까요오
	 * 나는 무엇을 위해 존재할 까요??
	 * @return
	 */
	public static boolean isGasOnlyOne() {
		String checkFileName;
		
		checkFileName = CommonUtil.isWindows()?SERVERINFO.WIN_ONLY_ONE_GAS:SERVERINFO.ONLY_ONE_GAS;
		
		File checkFile = new File(checkFileName);
		
		return checkFile.exists();
	}
	
	public static Map<String, Object> jsonToMap(String jsonString){
		ObjectMapper mapper = new ObjectMapper();
		Map<String, Object> map = new HashMap<String, Object>();
		
		//Convert JSON string to map
		try {
			map = mapper.readValue(jsonString, new TypeReference<Map<String, Object>>(){});
		} catch (JsonParseException e) {
			e.printStackTrace();
			logger.error("jsonToMap Error" + e.getMessage());
		} catch (JsonMappingException e) {
			e.printStackTrace();
			logger.error("jsonToMap Error" + e.getMessage());
		} catch (IOException e) {
			e.printStackTrace();
			logger.error("jsonToMap Error" + e.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("jsonToMap Error" + e.getMessage());
		}
		return map;
	}
	
	public static String getStrTypeCurTime() {
		Date now = new Date();
		String currentTime = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss").format(now);
		
		return currentTime;
	}
	
	public static String getStrTypeCurTimeAfterSec(int seconds) {
		Date today = new Date();
		Calendar cal = Calendar.getInstance();
		cal.setTime(today);
		cal.add(Calendar.SECOND, + seconds);
		
		SimpleDateFormat sdformat = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss"); 
		String EndTime = sdformat.format(cal.getTime());  
		
		return EndTime;
	}
	
	public static String getStrTypeTimeWithLongType(long seconds) {
		Date today = new Date();
		Calendar cal = Calendar.getInstance();
		cal.setTimeInMillis(seconds);
		
		SimpleDateFormat sdFormat = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss");
		return sdFormat.format(cal.getTime());
	}
	
	public static String convertTimeToKakaoTime(String strTime) throws Exception {
		String result = "";
		SimpleDateFormat sdFormat = new SimpleDateFormat("yyyyMMddHHmmss");
		
		Date n = sdFormat.parse(strTime);
		SimpleDateFormat sdFormat2 = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss");
		String tmpTime = sdFormat2.format(n);
		result = tmpTime.split("_")[0] + "T" + tmpTime.split("_")[1] + "Z";
		return result;
	}
	// String 타입의 두 날짜를 비교하기
	// -1 면 startTime 이 크다.
	// 0 이면 두 시간은 같다
	// 1 이면 endTime이 더 크다.
	public static int compareStringTypeTime(String startTime, String endTime) {
		try {
			SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss");
			Date start_time = dateFormat.parse(startTime);
			Date end_time = dateFormat.parse(endTime);
			
			if(start_time.getTime() < end_time.getTime()) return 1;
			
			if(start_time.getTime() > end_time.getTime()) return -1;
		}catch(Exception e) {
			return -2;
		}
		return 0;
	}
	
	/**
	 * HTTP Header의 authorization 필드 비교 하여 client id : secret이 적합한것인지 확인
	 * @param authorization
	 * @return
	 */
	public static int validateHeaderAuth(String authorization) {
		String splitStr[] = authorization.split(" ");
		logger.debug("[validateHeaderAuth]-[splitStr length]=" + splitStr.length);
		// splitStr length 가 2가 아니면 유효하지 않은 authorization 값
//		if(splitStr.length != 2) return KAKAO_RESULT.FAIL_INVALID_CLIENT;
		
		logger.debug("[validateHeaderAuth]-[splitStr[0]]=" + splitStr[0]);
		// splitStr이 Basic이 아니어도 에러
//		if(!splitStr[0].equals("Basic")) return KAKAO_RESULT.FAIL_INVALID_CLIENT;
		
		String encodeStr = splitStr.length != 2?splitStr[0]:splitStr[1];
		
		//Base64 디코드
		String decodeStr = new String(Base64.getDecoder().decode(encodeStr));
		
		splitStr = decodeStr.split(":");
		logger.debug("[validateHeaderAuth]-[splitStr length 2]=" + splitStr.length);
		// splitStr length 가 2가 아니면 유효하지 않은 authorization 값
		if(splitStr.length != 2) return KAKAO_RESULT.FAIL_INVALID_CLIENT;
		
		logger.info("request   client id = " + splitStr[0]);
		logger.info("icontrols client id = " + splitStr[1]);
		logger.info("request   secret = " + SERVERINFO.KAKAO_HEADER_CLIENT_ID);
		logger.info("icontrols secret = " + SERVERINFO.KAKAO_HEADER_CLIENT_SECRET);
		if(splitStr[0].equals(SERVERINFO.KAKAO_HEADER_CLIENT_ID) && splitStr[1].equals(SERVERINFO.KAKAO_HEADER_CLIENT_SECRET)) {
			return KAKAO_RESULT.SUCCESS;
		}
		
		return KAKAO_RESULT.FAIL_INVALID_CLIENT;
	}
	
	public static String getClientIP(HttpServletRequest request) {
		String ip = request.getHeader("X-FORWARDED-FOR");
		
		if(ip == null || ip.length() == 0) {
			ip = request.getHeader("Proxy-Client-IP");
		}
		
		if(ip == null || ip.length() == 0) {
			ip = request.getHeader("WL-Proxy-Client-IP"); //웹 로직
		}
		
		if(ip == null || ip.length() == 0) {
			ip = request.getRemoteAddr();
		}
		
		return ip;
	}
	
	public static String readFile(String filePath) {
        BufferedReader br = null;
        StringBuffer sb = new StringBuffer(); // 테스트용 변수
         try {
            br = new BufferedReader(new FileReader(filePath));
            String line = null;
            
            while ((line = br.readLine()) != null) {
                //System.out.println(line);
            	// 주석부분은 읽지 않는다.
            	if(line.length() > 0 && line.charAt(0) == '#') continue;
                
            	sb.append(line);
            }
 
        } catch (IOException ioe) {
            System.out.println(ioe.getMessage());
            sb = new StringBuffer();
        } finally {
            try { 
                if (br!=null) 
                    br.close(); 
            } catch (Exception e) {}
        }
         
         return sb.toString();
    }
	
	public static String mapToString(Map<String, Object> retMap) {
		String jsonStr = "";
		
		ObjectMapper mapper = new ObjectMapper(); 
		try { 
			// convert map to JSON string 
			jsonStr = mapper.writeValueAsString(retMap); 
			// compact-print 
//			jsonStr = mapper.writerWithDefaultPrettyPrinter().writeValueAsString(retMap); 
			// pretty-print 
			} catch (JsonProcessingException e) { e.printStackTrace(); }
		
		return jsonStr;
	}
	
	public static int getAirQuaility(String type, int value) {
		switch(type) {
			case "co2":
				if(value < 1001)
					return 0;
				else if(value <= 1001 && value < 1501)
					return 1;
				else if(value <= 1501 && value < 3001)
					return 2;
				else return 3;
			case "pm10":
				if(value < 31)
					return 0;
				else if(value <= 31 && value < 81)
					return 1;
				else if(value <= 81 && value < 150)
					return 2;
				else return 3;
			case "pm2_5":
				if(value < 16)
					return 0;
				else if(value <= 16 && value < 36)
					return 1;
				else if(value <= 36 && value < 76)
					return 2;
				else return 3;
		}
		
		return 0;
	}
}
