package com.icontrols.kakao.config;

import java.util.concurrent.Executor;

import javax.annotation.Resource;

import org.springframework.aop.interceptor.AsyncUncaughtExceptionHandler;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.AsyncConfigurer;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

@Configuration
@EnableAsync
public class AsyncConfig implements AsyncConfigurer{

	/** 샘플 기본 Thread 수 */
    private static int TASK_SAMPLE_CORE_POOL_SIZE = 20;
    /** 샘플 최대 Thread 수 */
    private static int TASK_SAMPLE_MAX_POOL_SIZE = 30;
    /** 샘플 QUEUE 수 */
    private static int TASK_SAMPLE_QUEUE_CAPACITY = 0;
    /** 샘플 Thread Bean Name */
    private static String EXECUTOR_SAMPLE_BEAN_NAME = "wallpadExecutor";
    /** 샘플 Thread */
    
    @Resource(name = "wallpadExecutor")
    private ThreadPoolTaskExecutor wallpadExecutor;

    @Bean(name = "wallpadExecutor")
    @Override
    public Executor getAsyncExecutor() {
        ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
        executor.setCorePoolSize(TASK_SAMPLE_CORE_POOL_SIZE);
        executor.setMaxPoolSize(TASK_SAMPLE_MAX_POOL_SIZE);
        executor.setQueueCapacity(TASK_SAMPLE_QUEUE_CAPACITY);
        executor.setBeanName(EXECUTOR_SAMPLE_BEAN_NAME);
        executor.initialize();
        return executor;
    }

	@Override
	public AsyncUncaughtExceptionHandler getAsyncUncaughtExceptionHandler() {
		// TODO Auto-generated method stub
		return new AsyncExceptionHandler();
	}
	
	/**
     * 샘플 Thread 등록 가능 여부
     *
     * @return 실행중인 task 개수가 최대 개수(max + queue)보다 크거나 같으면 false
     */
    public boolean isTaskExecute() {
        boolean rtn = true;
 
        System.out.println("EXECUTOR_SAMPLE.getActiveCount() : " + wallpadExecutor.getActiveCount());
 
        // 실행중인 task 개수가 최대 개수(max + queue)보다 크거나 같으면 false
        if (wallpadExecutor.getActiveCount() >= (TASK_SAMPLE_MAX_POOL_SIZE + TASK_SAMPLE_QUEUE_CAPACITY)) {
            rtn = false;
        }
 
        return rtn;
    }
 
    /**
     * 샘플 Thread 등록 가능 여부
     *
     * @param createCnt : 생성 개수
     * @return 실행중인 task 개수 + 실행할 개수가 최대 개수(max + queue)보다 크면 false
     */
    public boolean isTaskExecute(int createCnt) {
        boolean rtn = true;
 
        // 실행중인 task 개수 + 실행할 개수가 최대 개수(max + queue)보다 크거나 같으면 false
        if ((wallpadExecutor.getActiveCount() + createCnt) > (TASK_SAMPLE_MAX_POOL_SIZE + TASK_SAMPLE_QUEUE_CAPACITY)) {
            rtn = false;
        }
 
        return rtn;
    }
    
    
    @Resource(name = "eventExecutor")
    private ThreadPoolTaskExecutor eventExecutor;
    
    
    /** 기본 Thread 수 */
    private static int EVENT_TASK_CORE_POOL_SIZE = 20;
    /** 최대 Thread 수 */
    private static int EVENT_TASK_MAX_POOL_SIZE = 30;
    /** QUEUE 수 */
    private static int EVENT_TASK_QUEUE_CAPACITY = 0;
    /** Thread Bean Name */
    private static String EVENT_EXECUTOR_BEAN_NAME = "wallpadExecutor";
    /** Thread */
    
    
    @Bean(name = "eventExecutor")
    @Qualifier // 타입충돌 방지
    public Executor taskExecutorEtc() {
        ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
        executor.setCorePoolSize(EVENT_TASK_CORE_POOL_SIZE);
        executor.setMaxPoolSize(EVENT_TASK_MAX_POOL_SIZE);
        executor.setQueueCapacity(EVENT_TASK_QUEUE_CAPACITY);
        executor.setBeanName(EVENT_EXECUTOR_BEAN_NAME);
        executor.initialize();
        return executor;
    }
    
    /**
     * 샘플 Thread 등록 가능 여부
     *
     * @return 실행중인 task 개수가 최대 개수(max + queue)보다 크거나 같으면 false
     */
    public boolean isEventTaskExecute() {
        boolean rtn = true;
 
        System.out.println("eventExecutor.getActiveCount() : " + eventExecutor.getActiveCount());
 
        // 실행중인 task 개수가 최대 개수(max + queue)보다 크거나 같으면 false
        if (eventExecutor.getActiveCount() >= (EVENT_TASK_MAX_POOL_SIZE + EVENT_TASK_QUEUE_CAPACITY)) {
            rtn = false;
        }
 
        return rtn;
    }
 
    /**
     * 샘플 Thread 등록 가능 여부
     *
     * @param createCnt : 생성 개수
     * @return 실행중인 task 개수 + 실행할 개수가 최대 개수(max + queue)보다 크면 false
     */
    public boolean isEventTaskExecute(int createCnt) {
        boolean rtn = true;
 
        // 실행중인 task 개수 + 실행할 개수가 최대 개수(max + queue)보다 크거나 같으면 false
        if ((eventExecutor.getActiveCount() + createCnt) > (EVENT_TASK_MAX_POOL_SIZE + EVENT_TASK_QUEUE_CAPACITY)) {
            rtn = false;
        }
 
        return rtn;
    }
}
