package com.icontrols.kakao.config;

import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.icontrols.kakao.common.SERVERINFO;
import com.icontrols.kakao.common.utils.CommonUtil;

public class DanziSettingConfig {
	private final Logger logger = LoggerFactory.getLogger(DanziSettingConfig.class);
	
	private String filePath;
	private String fileName;
	
	private String applicationInfoStr;
	private boolean configCompleted = false;
	private Map<String, String> configInfo;
	
	public DanziSettingConfig() {
		super();
		logger.info("[단지 설정 클래스 실행]");
		
//		filePath = CommonUtil.isWindows()?"D:\\temp\\kakao_bepo\\":"/home/valley/kakao/";
		filePath = CommonUtil.isWindows()?"D:\\temp\\kakao_bepo\\":"/service/kakao/";
		fileName = "application_setting_info.dat";
		configCompleted = true;
		configInfo = new HashMap<String, String>();
	}
	
	public DanziSettingConfig getApplicationInfoStr() throws Exception{
		logger.info("[1. 어플리케이션 설정정보 가져오기] - filePath = " + filePath+fileName);
		
		applicationInfoStr = CommonUtil.readFile(filePath+fileName).trim();
		logger.info(applicationInfoStr);
		
		if(applicationInfoStr.length() == 0) {
			logger.error("[1. 어플리케이션 설정정보 가져오기 에러]");
			configCompleted = false;
			return this;
		}
		return this;
		
	}
	public DanziSettingConfig setConfigInfo() {
		logger.info("[2. 설정정보 매핑하기]");
		
		if(!configCompleted) {
			logger.error("[DanziSettingConfig ERROR] - setConfigInfo 이전단계 성공하지 않음");
			return this;
		}
		try {
			String[] infoBuf = applicationInfoStr.split(",");
			
			for(String str:infoBuf) {
				logger.debug("str -> " + str);
				
				String key = str.split("=")[0].trim();
				String val = str.split("=")[1].trim();
				
				configInfo.put(key, val);
			}
		}catch(Exception e) {
			logger.error("pDanziSettingConfig ERROR] - setConfigInfo 알수 없는 에러 발생 -" +e.getMessage());
			configCompleted = false;
		}
		
		return this;
	}
	
	public DanziSettingConfig setServerInfoALL() {
		logger.info("[3. SERVERINFO.java 값 수정]");
		if(!configCompleted) {
			logger.error("[DanziSettingConfig ERROR] - setServerInfo 이전단계 성공하지 않음");
			return this;
		}
		
		try {
			String[] infoBuf = applicationInfoStr.split(",");
			
			for(String str:infoBuf) {
				String key = str.split("=")[0].trim();
				
				setServerInfo(key);
			}
		}catch(Exception e) {
			logger.error("[DanziSettginConfig ERROR] - setServerInfoALL 알 수 없는 에러 발생 -" + e.getMessage());
			
		}
		
		return this;
	}
	
	public void setServerInfo(String key) {
		logger.info("[3. SERVERINFO.java 값 수정 각각]");
		if(!configCompleted) {
			logger.error("[DanziSettingConfig ERROR] - setServerInfo 이전단계 성공하지 않음");
			return;
		}
		
		try{
			switch(key) {
				case "aptCode":
					SERVERINFO.danzi_code = configInfo.get("aptCode");
					break;
				case "danziInfo":
					SERVERINFO.danzi_info = configInfo.get("danziInfo");
					break;
				case "settingVersion":
					SERVERINFO.application_version = configInfo.get("settingVersion");
					break;
				case "releaseVersion":
					SERVERINFO.release_version = configInfo.get("releaseVersion");
					break;
				case "danziServerExip":
					SERVERINFO.danziServerExIp = configInfo.get("danziServerExip");
					break;
				case "wallpadSocketPort":
					SERVERINFO.clientNewPort = Integer.parseInt(configInfo.get("wallpadSocketPort"));
					break;
				case "eventSocketPort":
					SERVERINFO.WalladReceivePort = Integer.parseInt(configInfo.get("eventSocketPort"));
					break;
				case "danziServerIP":
					SERVERINFO.DanjiServerIP = configInfo.get("danziServerIP");
					break;
				case "sendKakaoEventURL":
					SERVERINFO.SendKakaoEventURL = configInfo.get("sendKakaoEventURL");
					break;
				case "kakaoHeaderClientId":
					SERVERINFO.KAKAO_HEADER_CLIENT_ID = configInfo.get("kakaoHeaderClientId");
					break;
				case "kakaoHeaderClientSecret":
					SERVERINFO.KAKAO_HEADER_CLIENT_SECRET = configInfo.get("kakaoHeaderClientSecret");
					break;
				case "kakaoEventClientId":
					SERVERINFO.KAKAO_EVENT_CLIENT_ID = configInfo.get("kakaoEventClientId");
					break;
				case "kakaoEventClientSecret":
					SERVERINFO.KAKAO_EVENT_CLIENT_SECRET = configInfo.get("kakaoEventClientSecret");
					break;
				case "kakaoIWallpadAccess" :
					SERVERINFO.KAKAO_I_WALLPAD_APPLY = configInfo.get("kakaoIWallpadAccess").equalsIgnoreCase("true")?true:false;
					break;
					
				default:
					logger.error("[DanziSettingConfig ERROR] - 해당 키 없음 -" + key);
			}
		}catch(Exception e) {
			logger.error("[DanziSettginConfig ERROR] - setServerInfo 알 수 없는 에러 발생 -" + e.getMessage());
		}
	}
	
	public Map<String, String> getConfigInfo(){
		return configInfo;
	}
	
	public boolean getConfigCompleted() {
		return configCompleted;
	}
	
	public void printCurrentServerInfo() {
		logger.info("[SERVERINFO.danzi_code] - " 			+ SERVERINFO.danzi_code
		+ "\r\n[SERVERINFO.danzi_info] - " 					+ SERVERINFO.danzi_info
		+ "\r\n[SERVERINFO.application_version] - " 		+ SERVERINFO.application_version
		+ "\r\n[SERVERINFO.release_version] - " 			+ SERVERINFO.release_version
		+ "\r\n[SERVERINFO.danziServerExIp] - " 			+ SERVERINFO.danziServerExIp
		+ "\r\n[SERVERINFO.clientNewPort] - " 				+ SERVERINFO.clientNewPort
		+ "\r\n[SERVERINFO.WalladReceivePort] - " 			+ SERVERINFO.WalladReceivePort
		+ "\r\n[SERVERINFO.DanjiServerIP] - " 				+ SERVERINFO.DanjiServerIP
		+ "\r\n[SERVERINFO.SendKakaoEventURL] - " 			+ SERVERINFO.SendKakaoEventURL
		+ "\r\n[SERVERINFO.KAKAO_HEADER_CLIENT_ID] - " 		+ SERVERINFO.KAKAO_HEADER_CLIENT_ID
		+ "\r\n[SERVERINFO.KAKAO_HEADER_CLIENT_SECRET] - " 	+ SERVERINFO.KAKAO_HEADER_CLIENT_SECRET
		+ "\r\n[SERVERINFO.KAKAO_EVENT_CLIENT_ID] - " 		+ SERVERINFO.KAKAO_EVENT_CLIENT_ID
		+ "\r\n[SERVERINFO.KAKAO_EVENT_CLIENT_SECRET] - " 	+ SERVERINFO.KAKAO_EVENT_CLIENT_SECRET);
	}
	
}
