package com.icontrols.kakao.controller;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.KEY_INFO;
import com.icontrols.kakao.common.SERVERINFO;
import com.icontrols.kakao.common.TYPE;
import com.icontrols.kakao.common.utils.CommonUtil;
import com.icontrols.kakao.common.utils.JWTUtils2;
import com.icontrols.kakao.service.AccessTokenService;
import com.icontrols.kakao.service.KakaoExtensionService;

@RestController
public class KakaoDanziController {
	
	private static final Logger logger = LoggerFactory.getLogger(KakaoDanziController.class);
	
	@Autowired
	KakaoExtensionService kakaoService;
	
	@Autowired
	AccessTokenService accessTokenService;
	/**
	 * 중계 서버에서 오는 요청을         분기 처리를 진행한다.
	 * @param header
	 * @return
	 */
	@RequestMapping(value = "/kakao/danzi", method = RequestMethod.POST, consumes = "application/json")
	public @ResponseBody Map<String, Object> getRequest(
														@RequestHeader HttpHeaders header,
														@RequestBody Map<String, Object> params,
														HttpServletResponse response, 
														HttpServletRequest request
														) throws Exception{
		
		logger.debug("[REQUEST] - "+ params.toString());
		
		int tokenValidResult = accessTokenService.validate((String)params.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
		
		if(tokenValidResult != KAKAO_RESULT.SUCCESS) {
			params.remove(KEY_INFO.HTTP_KEY_AUTHORIZATION);
			if(params.containsKey(KEY_INFO.HTTP_KEY_BODY)) params.remove(KEY_INFO.HTTP_KEY_BODY);
			
			params.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(tokenValidResult));
			
			return params;
		}
		
		// 받은 요청에 대해 반환 해주는 결과 JSON 객체
		Map<String, Object> retMap = null;
		try {
			//Request Parameter에서  type값을 통해 분기를 태운다. 
			String reqType = (params.containsKey(KEY_INFO.HTTP_KEY_TYPE))?(String) params.get(KEY_INFO.HTTP_KEY_TYPE):"";
			logger.info("[REQUEST INBOUD] reqType = " + reqType);
			
			if(reqType.equals(TYPE.REQ_TYPE.GRANTAUTH.getTypeName())) {
				// grantauth 인 경우에만 헤더의 인증 필드를 검증한다.
				if(!header.containsKey(SERVERINFO.HEADER_AUTH)) {
					params.remove(KEY_INFO.HTTP_KEY_BODY);
					params.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_CLIENT));
					retMap = params;
				}else {
//					String authorization =  request.getHeader(SERVERINFO.HEADER_AUTH);
//					logger.info("[Authorization] - "+authorization);
//					int validation_auth = CommonUtil.validateHeaderAuth(authorization);
//					logger.info("[validation_auth]-" + validation_auth);
//					if(validation_auth != KAKAO_RESULT.SUCCESS) {
//						params.remove(KEY_INFO.HTTP_KEY_BODY);
//						params.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_CLIENT));
//						logger.error("[Authorization ERROR - ] - ["+authorization+"] - " + KAKAO_RESULT.FAIL_INVALID_CLIENT);
//						retMap = params;
//					}else {
//						// 카카오 홈과 유저의 인증 완료시 카카오에서 전송해주는 API TYPE
//						// 건내받은 토큰 정보를 DB에 저장해야 한다.
//						retMap = kakaoService.request_grantAuthorization(params);
//					}
					retMap = kakaoService.request_grantAuthorization(params);
				}
			} else if(reqType.equals(TYPE.REQ_TYPE.DISCOVER.getTypeName())) {
				// 월패드와 연결된 기기목록들을 요청하는 API TYPE
				retMap = kakaoService.request_discover(params);
			} else if(reqType.equals(TYPE.REQ_TYPE.QUERYSTATE.getTypeName())) {
				// 월패드와 연결된 기기중 endpointIds에 포함된 기기의 상태값을 확인한다.
				retMap = kakaoService.request_queryState(params);
			} else if(reqType.equals(TYPE.REQ_TYPE.UPDATESTATE.getTypeName())) {
				logger.debug("Update_state --> params \r\n" + params.toString());
				// 월패드와 연결된 기기의 상태를 번경한다. 
				retMap = kakaoService.request_updateState(params);
			}else {
				// 정의되지않은 type이 들어왔을경우.
				params.remove(KEY_INFO.HTTP_KEY_BODY);
				params.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_REQ));
				retMap = params;
			}
		}catch(Exception e) {
			logger.error("[Unknown ERROR]" + e.getMessage());
			logger.error(e.getStackTrace().toString());
			e.printStackTrace();
			retMap = KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL);
		}finally {
			retMap.remove(KEY_INFO.HTTP_KEY_AUTHORIZATION);
		}
		
		logger.info("[RESULT] - " + CommonUtil.mapToString(retMap));
		return retMap;
	}
}
