package com.icontrols.kakao.controller;

import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.KEY_INFO;
import com.icontrols.kakao.common.SERVERINFO;
import com.icontrols.kakao.common.TYPE;
import com.icontrols.kakao.common.utils.CommonUtil;
import com.icontrols.kakao.common.utils.JWTUtils2;
import com.icontrols.kakao.service.AccessTokenService;
import com.icontrols.kakao.service.KakaoExtensionService;
import com.icontrols.kakao.socket.wallpad.KakaoMsgHandler;

@RestController
public class TestController {
	
	private static final Logger logger = LoggerFactory.getLogger(TestController.class);
	
	@Autowired
	KakaoExtensionService kakaoService;
	
	@Autowired
	AccessTokenService accessTokenService;
	
	@Resource(name="kakaoMsgHandler")
	KakaoMsgHandler kMsgHandler;
	
	/**
	 * 중계 서버에서 오는 요청을         분기 처리를 진행한다.
	 * @param header
	 * @return
	 */
	@RequestMapping(value = "/test/Discover", method = RequestMethod.POST, consumes = "application/json")
	public @ResponseBody Map<String, Object> getRequest(
														@RequestHeader HttpHeaders header,
														@RequestBody Map<String, Object> params,
														HttpServletResponse response, 
														HttpServletRequest request
														) throws Exception{
		logger.debug("[REQUEST] - "+ params.toString());
		// 받은 요청에 대해 반환 해주는 결과 JSON 객체
		Map<String, Object> retMap = null;
		try {
			// 월패드와 연결된 기기목록들을 요청하는 API TYPE
			retMap = kakaoService.request_discover_test(params);
		}catch(Exception e) {
			logger.error("[Unknown ERROR]" + e.getMessage());
			logger.error(e.getStackTrace().toString());
			e.printStackTrace();
			retMap = KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL);
		}finally {
			retMap.remove(KEY_INFO.HTTP_KEY_AUTHORIZATION);
		}
		
		logger.info("[RESULT] - " + CommonUtil.mapToString(retMap));
		return retMap;
	}
	
	@RequestMapping(value = "/test/airconAttr", method = RequestMethod.POST, consumes = "application/json")
	public @ResponseBody Map<String, Object> getRequestAirConAttr(
														@RequestHeader HttpHeaders header,
														@RequestBody Map<String, Object> params,
														HttpServletResponse response, 
														HttpServletRequest request
														) throws Exception{
		logger.debug("[REQUEST] - "+ params.toString());
		// 받은 요청에 대해 반환 해주는 결과 JSON 객체
		Map<String, Object> retMap = null;
		try {
			// 월패드와 연결된 기기목록들을 요청하는 API TYPE
			int dong = Integer.parseInt((String)params.get("dong"));
			int ho = Integer.parseInt((String)params.get("ho"));
			retMap = kMsgHandler.getAirconCapability(dong, ho);
		}catch(Exception e) {
			logger.error("[Unknown ERROR]" + e.getMessage());
			logger.error(e.getStackTrace().toString());
			e.printStackTrace();
			retMap = KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL);
		}finally {
			retMap.remove(KEY_INFO.HTTP_KEY_AUTHORIZATION);
		}
		
		logger.info("[RESULT] - " + CommonUtil.mapToString(retMap));
		return retMap;
	}
	
	@RequestMapping(value = "/test/airconTemp", method = RequestMethod.POST, consumes = "application/json")
	public @ResponseBody Map<String, Object> getRequestAirConTemp(
														@RequestHeader HttpHeaders header,
														@RequestBody Map<String, Object> params,
														HttpServletResponse response, 
														HttpServletRequest request
														) throws Exception{
		logger.debug("[REQUEST] - "+ params.toString());
		// 받은 요청에 대해 반환 해주는 결과 JSON 객체
		Map<String, Object> retMap = null;
		try {
			// 월패드와 연결된 기기목록들을 요청하는 API TYPE
			int dong = Integer.parseInt((String)params.get("dong"));
			int ho = Integer.parseInt((String)params.get("ho"));
			String endpointId = (String)params.get("endpointId");
			retMap = kMsgHandler.getAirconEnableTemperature(dong, ho, endpointId);
		}catch(Exception e) {
			logger.error("[Unknown ERROR]" + e.getMessage());
			logger.error(e.getStackTrace().toString());
			e.printStackTrace();
			retMap = KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL);
		}finally {
			retMap.remove(KEY_INFO.HTTP_KEY_AUTHORIZATION);
		}
		logger.info("[RESULT] - " + CommonUtil.mapToString(retMap));
		return retMap;
	}
}
