package com.icontrols.kakao.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.KEY_INFO;
import com.icontrols.kakao.common.utils.JWTUtils2;
import com.icontrols.kakao.entity.DonghoInfo;
import com.icontrols.kakao.entity.UserInfo;
import com.icontrols.kakao.repo.DonghoInfoRepository;
import com.icontrols.kakao.repo.UserInfoRepository;
import com.icontrols.kakao.service.AccessTokenService;

@Service
public class AccessTokenServiceImpl implements AccessTokenService {
	private final Logger logger = LoggerFactory.getLogger(AccessTokenServiceImpl.class);
	
	
//	private final String ERROR_CODE_INTERNAL = "InternalServerError";
//	private final String ERROR_UNLINK_SERVICE = "UnlinkService";
//	private final String ERROR_TOKEN_NONE_EXIST = "TokenNoneExistent";
//	private final String ERROR_ALREADY_CREATED_TOKEN = "AlreadyCreatedToken";
	
	@Autowired
	DonghoInfoRepository donghoRepo;
	
	@Autowired
	UserInfoRepository userInfoRepo;
	
	@Override
	public boolean isTokenExpired(String accessToken) throws Exception {
		
		return true;
	}
	@Override
	public int validate(String accessToken) throws Exception {
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		// 1. db에 존재하는 accessToken인지 확인
		List<UserInfo> uList = userInfoRepo.findByAccessToken(accessToken);
		
		
		// 1.1 존재하지 않는 토큰이면 INVALID 토큰
		if(uList.size() == 0) {
			logger.error("[ERROR] - 존재하지 않는 토큰 - " + accessToken);
			return KAKAO_RESULT.FAIL_INVALID_TOKEN;
		}
		
		
		// 1.2 삭제된 토큰이면 UNLINK 토큰
		UserInfo userInfo = uList.get(0);
		logger.info("[validate] - " + userInfo.toString());
		if(userInfo.getDeleteFlag().equals("Y")) {
			if(!userInfo.getDeleteType().equalsIgnoreCase("expiredToken")) {
				logger.error("[ERROR] - 연동 해지 토큰 - " + accessToken);
				return KAKAO_RESULT.FAIL_UNLINKEXTENSION_SERVICE;
			}
		}
		
		// 2. JWT 검증
		int token_vali = JWTUtils2.validateToken(accessToken);
		
		if(token_vali == KAKAO_RESULT.FAIL_EXPIREDTOKEN) {
			logger.info("[ERROR] - 인 증 토 큰 만 료 - " + accessToken);
			return KAKAO_RESULT.FAIL_EXPIREDTOKEN;
		}else if(token_vali == KAKAO_RESULT.FAIL_INVALID_TOKEN) {
			logger.error("[ERROR] - 유 효 하 지 않 는 토 큰 - " + accessToken);
			return KAKAO_RESULT.FAIL_INVALID_TOKEN;
		}
		
		return KAKAO_RESULT.SUCCESS;
	}
	
	@Override
	public Map<String, Object> create(Map<String, Object> param) throws Exception {
		Map<String, Object> retMap = new HashMap<String, Object>();
		try {
			String accessToken = (String) param.get("accessToken");
			String refreshToken = (String) param.get("refreshToken");
			
			List<UserInfo> uList = userInfoRepo.findByAccessTokenOrRefreshToken(accessToken, refreshToken);
			
			if(uList.size() > 0) {
				Map<String, Object> errMap = new HashMap<String, Object>();
				errMap.put("code", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getResultCode());
				errMap.put("message", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getMessage());
//				errMap.put("message", ERROR_ALREADY_CREATED_TOKEN);
				
				retMap.put("error", errMap);
				logger.error("[AccessTokenService]-[CREATE] - [ERROR] - ALREADY CREATED TOKEN");
				return retMap;
			}
			String dong_ho = JWTUtils2.getHomeIdFromToken(accessToken);
			String dong = dong_ho.split("/")[0];
			String ho = dong_ho.split("/")[1];
			
			DonghoInfo donghoInfo = new DonghoInfo(dong_ho, dong, ho);
			donghoRepo.save(donghoInfo);

			
			UserInfo userInfo = new UserInfo(accessToken, refreshToken, donghoInfo);
			userInfoRepo.save(userInfo);
		
		}catch(Exception e) {
			Map<String, Object> errMap = new HashMap<String, Object>();
			errMap.put("code", KAKAO_RESULT.CODE.FAIL_INTERNAL.getResultCode());
			errMap.put("message", KAKAO_RESULT.CODE.FAIL_INTERNAL.getMessage());
			
			retMap.put("error", errMap);
			logger.error("[AccessTokenService]-[CREATE] - [ERROR] - " + e.getMessage());
			
			e.printStackTrace();
		}finally {
			logger.info("AccessTokenService [CREATE] - [END] - " + retMap.toString());
		}
		
		return retMap;
	}

	@Override
	public Map<String, Object> refresh(Map<String, Object> param) throws Exception {
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		try {
			String prevRefreshToken = (String) param.get("prevRefreshToken");
			String newAccessToken = (String) param.get("newAccessToken");
			String newRefreshToken = (String) param.get("newRefreshToken");
			
			String dong_ho = JWTUtils2.getHomeIdFromToken(newAccessToken);
			String dong = dong_ho.split("/")[0];
			String ho = dong_ho.split("/")[1];
			
			// 1. prevRefreshToken 과 같은 refreshToken 가진 DonghoInfo 조회
			List<UserInfo> userInfoList = userInfoRepo.findByRefreshToken(prevRefreshToken);
			// 1.1 DonghoInfo가 없으면 ERROR Return
			if(userInfoList.size() == 0) {
				Map<String, Object> errMap = new HashMap<String, Object>();
				errMap.put("code", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getResultCode());
				errMap.put("message", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getMessage());
//				errMap.put("message", ERROR_TOKEN_NONE_EXIST);
				
				retMap.put("error", errMap);
				logger.error("[AccessTokenService]-[REFRESH] - [ERROR] - TOKEN NONE EXIST1");
				return retMap;
			}
			
			// 2. newAccessTOken, newRefreshToken으로 대체
			UserInfo userInfo = userInfoList.get(0);
			logger.debug("prev dongho = " + userInfo.getDonghoInfo().getDonghoId() + "_ NEW dongho = " + dong_ho);
			if(!userInfo.getDonghoInfo().getDonghoId().equals(dong_ho)) {
				Map<String, Object> errMap = new HashMap<String, Object>();
				errMap.put("code", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getResultCode());
				errMap.put("message", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getMessage());
//				errMap.put("message", ERROR_TOKEN_NONE_EXIST);
				
				retMap.put("error", errMap);
				logger.error("[AccessTokenService]-[REFRESH] - [ERROR] - TOKEN NONE EXIST2");
				return retMap;
			}
			
			if(userInfo.getDeleteFlag().equals("Y")) {
				//토큰이 삭제된 이유가 토큰 만료면 에러가 아니다.
				if(!userInfo.getDeleteType().equalsIgnoreCase("expiredToken")) {
					Map<String, Object> errMap = new HashMap<String, Object>();
					errMap.put("code", KAKAO_RESULT.CODE.FAIL_UNLINKEXTENSION_SERVICE.getResultCode());
					errMap.put("message", KAKAO_RESULT.CODE.FAIL_UNLINKEXTENSION_SERVICE.getMessage());
//				errMap.put("message", ERROR_UNLINK_SERVICE);
					
					retMap.put("error", errMap);
					logger.error("[AccessTokenService]-[REFRESH] - [ERROR] - UNLINK TOKEN - " + userInfo.getDeleteType());
					return retMap;
				}
			}
			// 기존 토큰은 삭제하고 새로운 토큰을 생성한다.
			userInfo.setDeleteFlag("Y");
			userInfo.setDeleteType("expiredToken");
			
			
			UserInfo newUserInfo = new UserInfo(newAccessToken, newRefreshToken, userInfo.getDonghoInfo());
			newUserInfo.setKakaoAccessToken(userInfo.getKakaoAccessToken());
			newUserInfo.setKakaoExpiresTime(userInfo.getKakaoExpiresTime());
			newUserInfo.setKakaoRefreshToken(userInfo.getKakaoRefreshToken());
			newUserInfo.setKakaoTokenCreateTime(userInfo.getKakaoTokenCreateTime());
			newUserInfo.setKakaoTokenExpireTime(userInfo.getKakaoTokenExpireTime());
			newUserInfo.setPrevUserId(userInfo.getId());
			
			// 3. 저장
			userInfoRepo.save(userInfo);
			userInfoRepo.save(newUserInfo);
		}catch(Exception e) {
			Map<String, Object> errMap = new HashMap<String, Object>();
			errMap.put("code", KAKAO_RESULT.CODE.FAIL_INTERNAL.getResultCode());
			errMap.put("message", KAKAO_RESULT.CODE.FAIL_INTERNAL.getMessage());
//			errMap.put("message", ERROR_CODE_INTERNAL);
			logger.error("[AccessTokenService] - [REFRESH] - [ERROR] - " + e.getMessage());
			retMap.put("error", errMap);
			e.printStackTrace();
		}finally {
			logger.info("AccessTokenService [REFRESH] - [END] - " + retMap.toString());
		}
		return retMap;
	}

	@Override
	public Map<String, Object> get(Map<String, Object> param) throws Exception {
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		try {
			String refreshToken = (String) param.get("refreshToken");
			
			String dong_ho = JWTUtils2.getHomeIdFromToken(refreshToken);
			String dong = dong_ho.split("/")[0];
			String ho = dong_ho.split("/")[1];
			
			
			// 1. prevRefreshToken 과 같은 refreshToken 가진 DonghoInfo 조회
			List<UserInfo> userList = userInfoRepo.findByRefreshToken(refreshToken);
			
			// 1.1 DonghoInfo가 없으면 ERROR Return
			if(userList.size() == 0) {
				Map<String, Object> errMap = new HashMap<String, Object>();
				errMap.put("code", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getResultCode());
				errMap.put("message", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getMessage());
//				errMap.put("message", ERROR_TOKEN_NONE_EXIST);
				
				retMap.put("error", errMap);
				logger.error("[AccessTokenService]-[REFRESH] - [ERROR] - TOKEN NONE EXIST");
				return retMap;
			}
			
			// 2. 유효한 토큰인지 해지된 토큰인지 확인한다.
			UserInfo userInfo = userList.get(0);
			// 호옥시이나아 같은 동호가 아닐 수도 있으니깐
			if(!userInfo.getDonghoInfo().getDonghoId().equals(dong_ho)) {
				Map<String, Object> errMap = new HashMap<String, Object>();
				errMap.put("code", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getResultCode());
				errMap.put("message", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getMessage());
//				errMap.put("message", ERROR_TOKEN_NONE_EXIST);
				
				retMap.put("error", errMap);
				logger.error("[AccessTokenService]-[REFRESH] - [ERROR] - TOKEN NONE EXIST");
				return retMap;
			}
			
			// 삭제된 사용자인지 확인 (기타 다른 이유로 삭제 된건지 확인)
			if(userInfo.getDeleteFlag().equals("Y")) {
				if(!userInfo.getDeleteType().equalsIgnoreCase("expiredToken")) {
					Map<String, Object> errMap = new HashMap<String, Object>();
					errMap.put("code", KAKAO_RESULT.CODE.FAIL_UNLINKEXTENSION_SERVICE.getResultCode());
					errMap.put("message", KAKAO_RESULT.CODE.FAIL_UNLINKEXTENSION_SERVICE.getMessage());
//				errMap.put("message", ERROR_UNLINK_SERVICE);
					
					retMap.put("error", errMap);
					logger.error("[AccessTokenService]-[REFRESH] - [ERROR] - UNLINK TOKEN - " + userInfo.getDeleteType());
					return retMap;
				}
			}else {
				Map<String, Object> body = new HashMap<String, Object>();
				
				body.put("accessToken", userInfo.getAccessToken());
				body.put("refreshToken", userInfo.getRefreshToken());
				
				retMap.put("body", body);
			}
		}catch(Exception e) {
			Map<String, Object> errMap = new HashMap<String, Object>();
			errMap.put("code", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getResultCode());
			errMap.put("message", KAKAO_RESULT.CODE.FAIL_INVALID_TOKEN.getMessage());
//			errMap.put("message", ERROR_TOKEN_NONE_EXIST);
			
			retMap.put("error", errMap);
		}finally {
			logger.info("AccessTokenService [GET] - [END] - " + retMap.toString());
		}
		return retMap;
	}
}
