package com.icontrols.kakao.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.KEY_INFO;
import com.icontrols.kakao.common.utils.CommonUtil;
import com.icontrols.kakao.common.utils.JWTUtils2;
import com.icontrols.kakao.config.AsyncConfig;
import com.icontrols.kakao.entity.ThingsInfo;
import com.icontrols.kakao.entity.UserInfo;
import com.icontrols.kakao.repo.DonghoInfoRepository;
import com.icontrols.kakao.repo.ThingsInfoRepository;
import com.icontrols.kakao.repo.UserInfoRepository;
import com.icontrols.kakao.service.AuthService;
import com.icontrols.kakao.service.KakaoExtensionService;
import com.icontrols.kakao.socket.wallpad.KakaoMsgHandler;
import com.icontrols.kakao.socket.wallpad.WallpadMsgSender;


@Service
public class KakaoExtensionServiceImpl implements KakaoExtensionService {
	private final Logger logger = LoggerFactory.getLogger(KakaoExtensionService.class);
	
	@Resource(name="asyncConfig")
	private AsyncConfig asyncConfig;
	
	@Resource(name="kakaoMsgHandler")
	KakaoMsgHandler kMsgHandler;
	
	@Resource(name="wallpadMsgSender")
	WallpadMsgSender wallpadMsgSender;
	
	@Autowired
	AuthService authService;
	
	@Autowired
	UserInfoRepository userInfoRepo;
	
	@Autowired
	DonghoInfoRepository donghoRepository;
	
	@Autowired
	ThingsInfoRepository thingsInfoRepository;
	
	@Override
	@Transactional
	public Map<String, Object> request_grantAuthorization(Map<String, Object> param) throws Exception {
		String dong_ho = JWTUtils2.getHomeIdFromToken((String)param.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
		int dong = Integer.parseInt(dong_ho.split("/")[0]);
		int ho = Integer.parseInt(dong_ho.split("/")[1]);
		
		Map<String, Object> body = (HashMap)param.get(KEY_INFO.HTTP_KEY_BODY);
		
		try {
			// 1. 토큰 조회
			List<UserInfo> userList = userInfoRepo.findByAccessToken((String)param.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
			
			// 1.1 조회된 토큰 없으면 INVALID TOKEN
			if(userList.size() == 0) {
				logger.error("[This  AccessToken is not EXIST!!]");
				
				param.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
				
				return param;
			}
			
			// 2. 인증 해지된 토큰인지 확인
			UserInfo userInfo = userList.get(0);
			
			if(userInfo.getDonghoInfo().getDonghoId().equals("Y")) {
				// error 208
				logger.error("[THIS AccessToken is UNLINKED TOKEN.]");
				 
				param.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNLINKEXTENSION_SERVICE));
				
				return param;
			}
			
			Map<String, Object> subsMap = new HashMap<String, Object>();
			
			subsMap.put("type", "subs");
			subsMap.put("dong", dong_ho.split("/")[0]);
			subsMap.put("ho", dong_ho.split("/")[1]);
			subsMap.put("devNum", "1");
			subsMap.put("company", "kakao");
			
			// 구독 신청
			authService.subscriptionProcess(subsMap);
			
			// 모바일 기기 등록
			authService.remoteRegisterNewDevice(subsMap);
			
			// 기기목록 조회 및 저장
			List<ThingsInfo> databaseThingsList = thingsInfoRepository.findByDongAndHo(dong_ho.split("/")[0], dong_ho.split("/")[1]);
			List<ThingsInfo> wallpadThingsList = kMsgHandler.getConnectDevices(dong, ho);
			
			if(databaseThingsList.size() == 0) {
				
				if(wallpadThingsList == null) {
					logger.error("[Auth Database Insert Error]");
					param.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
					return param;
				}else {
					thingsInfoRepository.saveAll(wallpadThingsList);
				}
				
			}
			
			
			userInfo.setKakaoAccessToken((String) body.get(KEY_INFO.BODY_KEY_ACCESS_TOKEN));
			userInfo.setKakaoRefreshToken((String) body.get(KEY_INFO.BODY_KEY_REFRESH_TOKEN));
			userInfo.setKakaoExpiresTime(Integer.toString((int)body.get(KEY_INFO.BODY_KEY_EXPIRES_IN)));
			userInfo.setKakaoTokenCreateTime(CommonUtil.getStrTypeCurTime());
			userInfo.setKakaoTokenExpireTime(CommonUtil.getStrTypeCurTimeAfterSec((int)body.get(KEY_INFO.BODY_KEY_EXPIRES_IN)));
			
			logger.debug(userInfo.toString());
			userInfoRepo.save(userInfo);
			
			logger.debug("userInfo saved complete " );
			
		}catch(Exception e) {
			logger.error("[Auth Database Insert Error]");
			param.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
			
			e.printStackTrace();
		}finally {
			param.remove(KEY_INFO.HTTP_KEY_BODY);
		}
		
		return param;
	}
	
	@Override
	public Map<String, Object> request_discover(Map<String, Object> param) throws Exception{
		String dong_ho = JWTUtils2.getHomeIdFromToken((String)param.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
		int dong = Integer.parseInt(dong_ho.split("/")[0]);
		int ho = Integer.parseInt(dong_ho.split("/")[1]);
		
		Map<String, Object> listMap = kMsgHandler.getDevicesStatusList(dong, ho);
		param.putAll(listMap);
		
		logger.info("[request_discover] - result = \r\n" + param);
		return param;
	}
	
	@Override
	public Map<String, Object> request_discover_test(Map<String, Object> param) throws Exception{
		int dong = Integer.parseInt((String)param.get("dong"));
		int ho = Integer.parseInt((String)param.get("ho"));
		logger.debug("[dong]-"+dong + "[ho]-" + ho);
		
		Map<String, Object> listMap = kMsgHandler.getDevicesStatusListTemp(dong, ho);
		param.putAll(listMap);
		
		logger.info("[request_discover] - result = \r\n" + param);
		return param;
	}
	
	@Override
	public Map<String, Object> request_queryState(Map<String, Object> param) throws Exception {
		String dong_ho = JWTUtils2.getHomeIdFromToken((String)param.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
		int dong = Integer.parseInt(dong_ho.split("/")[0]);
		int ho = Integer.parseInt(dong_ho.split("/")[1]);
		
		Map <String, Object> body = (HashMap)param.get(KEY_INFO.HTTP_KEY_BODY);
		
		List<String> endpointIds = body.containsKey(KEY_INFO.BODY_KEY_ENDPOINT_IDS)?(ArrayList)body.get(KEY_INFO.BODY_KEY_ENDPOINT_IDS):null;
		List<String> unreachableIds = new ArrayList<String>();
		Map<String, Object> allDeviceMap = null;
		
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		logger.debug("[QUERYSTATE] - " + endpointIds);
		
		try {
			if(endpointIds == null) {
				param.remove(KEY_INFO.HTTP_KEY_BODY);
				
				Map<String, Object> errMap = new HashMap<String, Object>();
				errMap.put(KEY_INFO.HTTP_KEY_ERROR,KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_REQ));

				param.putAll(errMap);
				logger.error("[ERROR] - INVALID REQ _01");
				return param;
			}
			
			allDeviceMap = (Map<String, Object>) kMsgHandler.getDevicesStatusList(dong, ho).get(KEY_INFO.HTTP_KEY_BODY);
			logger.debug("[DEViCES] - " + allDeviceMap.toString());
			
			for(String endpointId:endpointIds) {
				// 1. db 에서 해당 endpointId가 존재하는지 확인한다.
				// 없으면 endpointId가 잘못된것 -203 에러를 뱉자.
				List<ThingsInfo> thingsInfoList = thingsInfoRepository.findByDongAndHoAndEndpointId(Integer.toString(dong), Integer.toString(ho), endpointId);
				if(thingsInfoList == null || thingsInfoList.size() == 0) {
					param.remove(KEY_INFO.HTTP_KEY_BODY);
					
					Map<String, Object> errMap = new HashMap<String, Object>();
					errMap.put(KEY_INFO.HTTP_KEY_ERROR,KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_NONEXIST));

					param.putAll(errMap);
					
					return param;
				}
				
				// 2. allDeviceMap에서 해당 endpointId 를 조회하자.
				// 없으면 연결이 해제된것 unreachable 배열에 추가하자
				Map<String, Object> state = new HashMap<String, Object>();
				
				if(!allDeviceMap.containsKey(endpointId)) {
					unreachableIds.add(endpointId);
					state.put("power", "off");
					logger.debug("[" + endpointId + "] - unreachableId");
				}else {
					state = (Map<String, Object>)((Map<String, Object>)allDeviceMap.get(endpointId)).get("state");
					logger.debug("["+endpointId+"] - state = " + state );
				}
				
				retMap.put(endpointId, state);
			}
			
			body.remove(KEY_INFO.BODY_KEY_ENDPOINT_IDS);
			body.putAll(retMap);
			param.put(KEY_INFO.HTTP_KEY_UNREACHABLEIDS, unreachableIds);
		}catch(Exception e) {
			param.remove(KEY_INFO.HTTP_KEY_BODY);
			
			Map<String, Object> errMap = new HashMap<String, Object>();
			errMap.put(KEY_INFO.HTTP_KEY_ERROR,KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));

			param.putAll(errMap);
			
			e.printStackTrace();
			return param;
		}
		
		return param;
	}
	
	
	@Override
	public Map<String, Object> request_updateState(Map<String, Object> param) throws Exception {
		String dong_ho = JWTUtils2.getHomeIdFromToken((String)param.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
		int dong = Integer.parseInt(dong_ho.split("/")[0]);
		int ho = Integer.parseInt(dong_ho.split("/")[1]);
		
		@SuppressWarnings("unchecked")
		Map <String, Object> body = (HashMap<String, Object>)param.get(KEY_INFO.HTTP_KEY_BODY);
		
		@SuppressWarnings("unchecked")
		Map<String, Object> desired = (HashMap<String, Object>)body.get(KEY_INFO.BODY_KEY_DESIRED);
		Map<String, Object> retMap = null;
		String endpointId = (String)body.get(KEY_INFO.BODY_KEY_ENDPOINT_ID);
		String capabilityName = (String)body.get(KEY_INFO.BODY_KEY_CAPABILITY_NAME);
		
		logger.info("[request_updateState]-[동:" + dong + "호:" + ho + "]-[" + endpointId + "]-[desired] - " + desired.toString());

		List<ThingsInfo> thingsInfoList = thingsInfoRepository.findByDongAndHoAndEndpointId(Integer.toString(dong), Integer.toString(ho), endpointId);
		
		if(thingsInfoList == null || thingsInfoList.size() == 0) {
			logger.error("[UPDATE-ERROR] - " + dong + "/" + ho + " - "+ endpointId + " is not exist");
			retMap = new HashMap<String, Object>();
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_NONEXIST));
		}else {
			retMap = kMsgHandler.sendDeviceAction(dong, ho, endpointId, capabilityName, desired);
		}
		
		logger.info("[RESPONSE]-[request_updateState]-[동:" + dong + "호:" + ho + "]-" + retMap.toString());
		
		if(retMap.containsKey(KEY_INFO.HTTP_KEY_ERROR)) {
			param.remove(KEY_INFO.HTTP_KEY_BODY);
			param.putAll(retMap);
		}else {
			body.clear();
			body.putAll(retMap);
			param.put(KEY_INFO.HTTP_KEY_BODY, body);
		}

		return param;
	}
}
