package com.icontrols.kakao.socket.utils;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.InetSocketAddress;
import java.net.Socket;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class iMapRequestSync {

	private static final Logger logger = LoggerFactory.getLogger(iMapRequestSync.class);

	public static final int APT_SERVER = 1;
	public static final int APT_LOBBY = 2;

	public static final int TYPE_REQUEST = 1;
	public static final int TYPE_NOTICE = 2;
	public static final int TYPE_RESPONSE = 3;

	public static final int COMM_COMPLITE = 1;
	public static final int COMM_ERROR = 2;
	public static final int COMM_DOING = 3;
	public static final int COMM_NONE = 4;

	public static final int SOCKET_BUFFER_SIZE = 100;
	public static final int SOCKET_READ_BUFFER = 100;

	int SOCKET_WAIT_COUNT = 30;
	int SOCKET_WAIT_TIME = 3000;

	long socket_time = 0;
	Socket isock;
	String ClientIP = "10.2.14.1";
	int ClientPort = 10000;

	boolean log_sw = true;
	private String iMapXML = "";
	private Document doc = null;

	NodeList items;
	int Svrtarget;

	boolean LogSW = true;

	String cmdDoc;

	void iMAPClientLog(String msg) {
		if (LogSW)
			logger.debug("iMAPClient :" + msg);
	}

	public iMapRequestSync(String svrIP, int svrPORT) 
	{
		ClientIP = svrIP;
		ClientPort     = svrPORT;
	}

	public void SetSocketWaitCount(int count) {
		SOCKET_WAIT_TIME = count;
	}

	public int doWork(String XML) {
		int retValue = 0;
		iMAPClientLog("Send XML : \n " + XML);
		boolean bRet = reqSocketCommunication(XML, ClientIP, ClientPort, SOCKET_WAIT_TIME); //실제 실행

		if (bRet == true) {
			retValue = 0; // 커넥션 정상
		} else {
			retValue = -1; // 커넥션 안됨
		}

		return retValue;
	}

	public String getResponseXML() {
		return iMapXML;
	}

	public Document getDocument() {
		return doc;
	}

	public boolean reqSocketCommunication(String XML, String SoIPa, int SoIPport, int TimeOut) {
		logger.debug("reqSocketCommunication");
		try {
			cmdDoc = XML + "\r\n"; // 보낼 XML 메시지

			// 소켓 설정
			isock = new Socket();
			isock.setSoLinger(true, 3000);
			isock.setReceiveBufferSize(SOCKET_BUFFER_SIZE * 1024);
			isock.setSendBufferSize(SOCKET_BUFFER_SIZE * 1024);
			isock.setSoTimeout(TimeOut);
			InetSocketAddress iSocAddr = new InetSocketAddress(SoIPa, SoIPport); 
			
			//소켓 연결 
			isock.connect(iSocAddr, SOCKET_WAIT_TIME);
			
			InputStream In = isock.getInputStream();
			BufferedReader XMLReader = new BufferedReader(new InputStreamReader(In, "euc-kr"));
			DataOutputStream outstream = new DataOutputStream(isock.getOutputStream());
			byte[] outBF = cmdDoc.getBytes("euc-kr");
			outstream.write(outBF);
			outstream.flush();

			iMapXML = "";
			String readbuff;
			int i = 0;
			boolean succ = false;
			socket_time = System.currentTimeMillis() + SOCKET_WAIT_TIME;
			while (System.currentTimeMillis() < socket_time) {
				if (XMLReader.ready()) {

					char[] indata = new char[SOCKET_READ_BUFFER * 1024];
					readbuff = "";

					while (XMLReader.ready()) {
						long lenofxML = XMLReader.read(indata);
						readbuff += new String(indata, 0, (int) lenofxML);

					}

					if (readbuff != null)
						iMapXML += readbuff;

					if (-1 < readbuff.indexOf("</imap>")) {
						succ = true;
						break;
					}
				} else {
					// imapAPIlog("LV1", "Socket sleep\n");
					try {
						Thread.sleep(100);
					} catch (Exception e) {
						logger.error("message : " + e);
						isock.close();
						return false;
					}
					i++;
				}
			}

			if (i == SOCKET_WAIT_COUNT || succ == false) {
				isock.close();
				return false;
			}
		} catch (Exception e) {
			logger.error("message : " + e);
			return false;
		}

		try {
			Thread.sleep(30);
			isock.close();
			SetXMLDoc(iMapXML); //변수 doc iMapXML 내용 선언해주는 과정 
			logger.debug(iMapXML);
			logger.debug("&&&&&&&&&&&&&&&&&    COMM_COMPLITE     &&&&&&&&&& 1");
			return true;
		} catch (IOException e) {
			logger.error("message : " + e);
		} catch (InterruptedException e) {
			logger.error("message : " + e);
		}

		return false;
	}

	private void SetXMLDoc(String XMLDoc) {

		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();

			iMapXML = XMLDoc;
			logger.debug("length :" + iMapXML.getBytes());
			InputStream istream = new ByteArrayInputStream(iMapXML.getBytes("UTF-8"));
			doc = builder.parse(istream);
			logger.debug(" iMapXML : " + iMapXML.length());
			if (doc == null) {
				try {
					for (int i = 0; i < 10; i++) {
						Thread.sleep(10);
						if (doc != null) {
							logger.debug(" doc parsing log wait  : " + i);

							break;
						}
					}
				} catch (Exception ex) {
					logger.error("message : " + ex);

				}
			}

		} catch (Exception e) {
			logger.error("message : " + e);
		}
	}


	public int SetItem(String name) {
		if (doc == null)
			return 0;
		items = doc.getElementsByTagName(name);
		return items.getLength();
	}

	public String GetAttributeVal(int index, String attname) {
		if (items == null)
			return null;
		int count = items.getLength();
		if (count < 1 || index - 1 > count)
			return null;
		Node item = items.item(index);
		NamedNodeMap attrs = item.getAttributes();
		for (int j = 0; j < attrs.getLength(); j++) {
			Node attr = attrs.item(j);

			if (attr.getNodeName().equalsIgnoreCase(attname))
				return attr.getNodeValue();
		}
		return null;
	}

	public String GetItemVal(int index) {
		int count = items.getLength();

		if (count < 1 || index - 1 > count)
			return null;
		Node item = items.item(index);

		return trimquotation(item.getTextContent());
	}

	protected String trimquotation(String sen) {
		sen = sen.replaceAll("\"", "");
		return sen.trim();
	}

	public void SetServerIPConfig(String svrIP, int setport) {
		ClientIP = svrIP;
		ClientPort = setport;
	}

	public String GetServerIPAddress() {
		return ClientIP;
	}
}
