package com.icontrols.kakao.socket.wallpad;

import java.net.URI;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

import javax.annotation.Resource;

import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.KAKAO_RESULT.CODE;
import com.icontrols.kakao.common.KEY_INFO;
import com.icontrols.kakao.common.SERVERINFO;
import com.icontrols.kakao.common.utils.BeanUtils;
import com.icontrols.kakao.common.utils.CommonUtil;
import com.icontrols.kakao.config.AsyncConfig;
import com.icontrols.kakao.entity.DonghoInfo;
import com.icontrols.kakao.entity.UserInfo;
import com.icontrols.kakao.repo.DonghoInfoRepository;
import com.icontrols.kakao.repo.UserInfoRepository;
import com.icontrols.kakao.service.AuthService;

@Service("kakaoEventHandler")
public class KakaoEventHandler {
	private final static Logger logger = LoggerFactory.getLogger(KakaoEventHandler.class);
	
	@Resource(name="asyncConfig")
	private AsyncConfig asyncConfig;
	
	@Resource(name="kakaoMsgHandler")
	private KakaoMsgHandler kakaoMsgHandler;
	
	@Resource(name="wallpadMsgSender")
	private WallpadMsgSender wallpadMsgSender;
	
	@Autowired
	private AuthService authService;
	
	@Async("eventExecutor")
	public CompletableFuture<Map<String, Object>> sendEventAsync(UserInfo userInfo, Map<String, Object> params) throws InterruptedException{
		Map<String, Object> retMap = new HashMap<String, Object>();
		try {
			// 인증토큰 검증
			String tokenStartTime = userInfo.getKakaoTokenCreateTime();
			String tokenExpiresTime = userInfo.getKakaoTokenExpireTime();
			String currentTime = CommonUtil.getStrTypeCurTime();
			
			int result = CommonUtil.compareStringTypeTime(currentTime, tokenExpiresTime);
			
			//boolean connFlag = true;
			// 현재 시간이 토큰 만료시간보다 크다면 토큰이 만료 됐다는 뜻.
			// 카카오 이벤트 서버에서 토큰을 갱신 받은 뒤 이벤트를 전송해야 한다.
//			if(result < 1) {
//				userInfo = sendKakaoTokenRefresh(userInfo);
//
//				if(userInfo == null) {
//					retMap.put("result", KAKAO_RESULT.FAIL_INTERNAL);
//					retMap.put("message", "ID-["+userInfo.getId() + "]-Fail to Token Refresh");
//					return CompletableFuture.completedFuture(retMap);
//				}
//				
//				params.put(KEY_INFO.HTTP_KEY_AUTHORIZATION, userInfo.getKakaoAccessToken());
//			}
			
			logger.debug("["+userInfo.getKakaoAccessToken()+"] - 유효한 토큰");
			
			// 이벤트 전송.
			Map<String, Object> sendResult = sendStatusResult(userInfo, params);
			
			int code = (int) sendResult.get("result");
			logger.info("[KAKAO EVENT SEND RESULT] - ["+code+"]");
			UserInfoRepository repo = null;
			switch(code) {
			case KAKAO_RESULT.FAIL_INTERNAL:
				break;
			case KAKAO_RESULT.FAIL_EXTENSION_SERVICE_TOKEN: //확장서비스 토큰 에러
				logger.error("["+userInfo.getKakaoAccessToken()+"] - 확장서비스 토큰 에러..삭제요청");
				repo = (UserInfoRepository)BeanUtils.getBean("userInfoRepository");
				userInfo.setDeleteFlag("Y");
				userInfo.setDeleteType(CODE.FAIL_INVALID_TOKEN.getMessage());
				repo.save(userInfo);
				
				retMap.put("result", KAKAO_RESULT.FAIL_INTERNAL);
				retMap.put("message", "ID-["+userInfo.getId() + "]-FAIL_EXTENSION_SERVICE_TOKEN");
				break;
			case KAKAO_RESULT.FAIL_INVALID_TOKEN: // 유효하지 않은 토큰, 연동 해제한 계정 토큰
				logger.error("[" + userInfo.getId() + "]-["+userInfo.getKakaoAccessToken()+"] - 유효하지 않은 토큰..삭제요청");
				repo = (UserInfoRepository)BeanUtils.getBean("userInfoRepository");
				userInfo.setDeleteFlag("Y");
				userInfo.setDeleteType(CODE.FAIL_INVALID_TOKEN.getMessage());
				repo.save(userInfo);
				
				retMap.put("result", KAKAO_RESULT.FAIL_INTERNAL);
				retMap.put("message", "ID-["+userInfo.getId() + "]-FAIL_INVALID_TOKEN");
				break;
			case KAKAO_RESULT.FAIL_UNLINKEXTENSION_SERVICE: // 서비스를 해지한 경우
				logger.error("["+userInfo.getKakaoAccessToken()+"] - 서비스 해제 토근..삭제요청");
				repo = (UserInfoRepository)BeanUtils.getBean("userInfoRepository");
				userInfo.setDeleteFlag("Y");
				userInfo.setDeleteType(CODE.FAIL_UNLINKEXTENSION_SERVICE.getMessage());
				repo.save(userInfo);
				
				retMap.put("result", KAKAO_RESULT.FAIL_UNLINKEXTENSION_SERVICE);
				retMap.put("message", "ID-["+userInfo.getId() + "]-FAIL_UNLINKEXTENSION_SERVICE");
				
				// 월패드 해지요청
				new UnlinkExtensionHandler(userInfo.getDonghoInfo().getDong(), userInfo.getDonghoInfo().getHo()).start();
				
				break;
			case KAKAO_RESULT.FAIL_EXPIREDTOKEN:  // 만료된 토큰, 토큰 재발급 뒤 재시도.
				// 재시도 할때는 따로 결과를 받지 않을려고.. 성공할때까지 계속 시도하는것도 비효율적.
				logger.info("["+userInfo.getKakaoAccessToken()+"]-인증토큰 만료 토큰 갱신 요청");
				UserInfo tmpUserInfo = sendKakaoTokenRefresh(userInfo);
				
				if(tmpUserInfo == null) {
					retMap.put("result", KAKAO_RESULT.FAIL_INTERNAL);
					retMap.put("message", "ID-["+tmpUserInfo.getId() + "]-Fail to Token Refresh Retry");
					return CompletableFuture.completedFuture(retMap);
				}
				
				logger.info("["+tmpUserInfo.getKakaoAccessToken()+"]-이벤트 재전송 시도");
				
				params.put(KEY_INFO.HTTP_KEY_AUTHORIZATION, tmpUserInfo.getKakaoAccessToken());
				
				sendResult = sendStatusResult(tmpUserInfo, params);
				
				code = (int) sendResult.get("result");
				
				retMap.put("result", code);
				
				if(code != KAKAO_RESULT.SUCCESS) {
					retMap.put("message", "ID-["+tmpUserInfo.getId() + "]-이벤트 재전송 실패 - [" + sendResult.get("message") + "]");
				}else {
					retMap.put("message", "ID-["+tmpUserInfo.getId() + "]-이벤트 재전송 성공");
				}
				break;
			case KAKAO_RESULT.SUCCESS:
				retMap.put("result", KAKAO_RESULT.SUCCESS);
				retMap.put("message", "ID-["+userInfo.getId() + "]-이벤트 전송 성공!");	
				break;
			}
		}catch(Exception e) {
			retMap.put("result", KAKAO_RESULT.FAIL_INTERNAL);
			logger.error("[ERROR] - " + e.getMessage());
			e.printStackTrace();
			retMap.put("message", "event send Error = " + e.getMessage());
		}
		
		return CompletableFuture.completedFuture(retMap);
	}
	
	/**
	 * 상태 변경 이벤트 발생시 이벤트 결과를 카카오 이벤트 서버에 전송한다. 
	 * 전송시 토큰이 만료 되면 [sendKakaoTokenRefresh] method를 호출하여 토큰을 갱신한뒤 다시 재시도 한다. 
	 * @param dong
	 * @param ho
	 * @param params
	 * @param reqType
	 */
	public Map<String, Object> sendStatusResult(UserInfo userInfo, Map<String, Object> params) {
		Map<String, Object> retMap = new HashMap<String, Object>();

		try {
			if(params == null) {
				retMap.put("result", KAKAO_RESULT.FAIL_INTERNAL);
				retMap.put("message", "params is null");
				logger.error("[sendStatusResult] - params is null");
				return retMap;
			}
			logger.info("[sendStatusResult] - START - " + CommonUtil.mapToString(params));
			
			String authorization = SERVERINFO.KAKAO_EVENT_CLIENT_ID+":"+SERVERINFO.KAKAO_EVENT_CLIENT_SECRET;
			String encoding = Base64.getEncoder().encodeToString(authorization.getBytes());

			//			logger.info("autho = " + authorization);
			//			logger.info("encoding = " + encoding);
			
			CloseableHttpClient httpClient = HttpClients.custom().setSSLHostnameVerifier(new NoopHostnameVerifier()).build();
			HttpComponentsClientHttpRequestFactory requestFactory = new HttpComponentsClientHttpRequestFactory();
			
			requestFactory.setHttpClient(httpClient);
			requestFactory.setConnectTimeout(10000);
			requestFactory.setReadTimeout(10000);
			RestTemplate restTemplate = new RestTemplate(requestFactory);

			HttpHeaders headers = new HttpHeaders();
			
			headers.setContentType(MediaType.APPLICATION_JSON);
			headers.set("Authorization", "Basic " + encoding);
			// 제어기기 별로 상태 이벤트 구분하여 처리하도록 해야 한다
			HttpEntity<Object> entity = new HttpEntity<Object>(params, headers);
			
			String url = SERVERINFO.SendKakaoEventURL + "/events";

			logger.info("SendStatusResult URL : " + url);
			logger.info("SendStatusResult header :  " + headers.toString());
			logger.info("SendStatusResult return value : " + params.toString());
			logger.info("sendStatusResult entity : " + entity.toString());
			String answer = restTemplate.postForObject(url, entity, String.class);
			
			logger.info("SendStatusResult answer :" + answer);
			
			// 결과에 error가 있으면 error에 맞게 대응해줘야 한다.
			Map<String, Object> resultMap = CommonUtil.jsonToMap(answer);
			
			if(resultMap.containsKey(KEY_INFO.HTTP_KEY_ERROR)) {
				//error 결과가 -201 이면 인증토큰이 만료 됐다는 뜻
				Map<String, Object> errMap = (Map<String, Object>)resultMap.get(KEY_INFO.HTTP_KEY_ERROR);
				int errorCode = Integer.parseInt((String)errMap.get(KEY_INFO.ERROR_KEY_CODE));
				String message = (String)errMap.get(KEY_INFO.ERROR_KEY_MESSAGE);
				
				logger.error("[KAKAO - ERROR] - [ CODE ] - " + errorCode + " || [message] - " + message );
				
				retMap.put("result", errorCode);
				retMap.put("message", message);
			}else {
				logger.info("EVENT SEND SUCCESS -- " + params.toString());
				
				retMap.put("result", KAKAO_RESULT.SUCCESS);
			}
		} catch (Exception e) {
			logger.error("SendStatusResult Exception e :" + e.toString());
			retMap.put("result", KAKAO_RESULT.FAIL_INTERNAL);
			retMap.put("message", e.getMessage());
		}
		
		return retMap;
	}
	
	/**
	 * 이벤트 전송시 토큰이 만료되면 해당 메서드가 호출된다.
	 * 이벤트 토큰 만료시 카카오 이벤트 토큰 서버에 토큰 갱신 요청을 보낸다.
	 * 토큰이 갱신 되면 이벤트 전송을 시도한다.
	 * @param dong
	 * @param ho
	 * @param params
	 */
	public UserInfo sendKakaoTokenRefresh(UserInfo userInfo){
		UserInfoRepository repo = (UserInfoRepository)BeanUtils.getBean("userInfoRepository");
		
		String refreshToken = userInfo.getKakaoRefreshToken();
		
		String authorization = SERVERINFO.KAKAO_EVENT_CLIENT_ID+":"+SERVERINFO.KAKAO_EVENT_CLIENT_SECRET;
		String encoding = Base64.getEncoder().encodeToString(authorization.getBytes());
		
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
		headers.add("Authorization", "Basic " + encoding);
		
		MultiValueMap<String, String> parameters = new LinkedMultiValueMap<String, String>();
		parameters.add("grant_type", "refresh_token");
		parameters.add("refresh_token", refreshToken);
		
		HttpEntity<MultiValueMap<String, String>> entity = new HttpEntity<>(parameters, headers);
		
		String url = SERVERINFO.SendKakaoEventURL +"/events/token";
		
//		String url = "https://kakaohome-api.i.kakao.com/events/token";
		logger.debug("[TOKEN REFRESH REQUEST] URL : " + url);
		logger.debug("[TOKEN REFRESH REQUEST] header : " + headers.toString());
		logger.debug("[TOKEN REFRESH REQUEST] return value : " + entity.toString());
		logger.debug("[TOKEN parameter] --" +parameters.toString());
		URI uri = URI.create(url);
		
		CloseableHttpClient httpClient = HttpClients.custom().setSSLHostnameVerifier(new NoopHostnameVerifier()).build();
		HttpComponentsClientHttpRequestFactory requestFactory = new HttpComponentsClientHttpRequestFactory();
		
		requestFactory.setHttpClient(httpClient);
		requestFactory.setConnectTimeout(10000);
		requestFactory.setReadTimeout(10000);

		String answer = "";
		try {
			RestTemplate restTemplate = new RestTemplate(requestFactory);
			answer = restTemplate.postForObject(url, entity, String.class);
			logger.info("[TOKEN REFRESH REQUEST] = " + answer);
			
			Map<String, Object> retMap = CommonUtil.jsonToMap(answer);
			
			if(retMap.containsKey(KEY_INFO.HTTP_KEY_ACCESS_TOKEN) && retMap.containsKey(KEY_INFO.HTTP_KEY_REFRESH_TOKEN) && retMap.containsKey(KEY_INFO.HTTP_KEY_EXPIRES_IN)) {
				userInfo.setKakaoAccessToken((String)retMap.get(KEY_INFO.HTTP_KEY_ACCESS_TOKEN));
				userInfo.setKakaoRefreshToken((String)retMap.get(KEY_INFO.HTTP_KEY_REFRESH_TOKEN));
				userInfo.setKakaoExpiresTime(Integer.toString((int)retMap.get(KEY_INFO.HTTP_KEY_EXPIRES_IN)));
				
				userInfo.setKakaoTokenCreateTime(CommonUtil.getStrTypeCurTime());
				userInfo.setKakaoTokenExpireTime(CommonUtil.getStrTypeCurTimeAfterSec(Integer.parseInt(userInfo.getKakaoExpiresTime())));
				
				repo.save(userInfo);
				
				logger.info("토큰 리프레시 성공 - " + userInfo.getDonghoInfo());
				return userInfo;
			}
		}catch(Exception e) {
				logger.error("[TOKEN REFRESH REQUEST ERROR] - " + e.getMessage());
				e.printStackTrace();
		}
		logger.error("토클 리프레시 실패" + userInfo.getDonghoInfo().getDong() + "_" + userInfo.getDonghoInfo().getHo());
		
		return null;
	}
}
