package com.icontrols.kakao.socket.wallpad;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.annotation.Resource;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.icontrols.kakao.common.ERROR_INFO;
import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.KEY_INFO;
import com.icontrols.kakao.common.SERVERINFO;
import com.icontrols.kakao.common.THINGS_INFO;
import com.icontrols.kakao.common.TYPE;
import com.icontrols.kakao.common.utils.BeanUtils;
import com.icontrols.kakao.common.utils.CommonUtil;
import com.icontrols.kakao.common.utils.NodeUtil;
import com.icontrols.kakao.common.utils.XmlUtil;
import com.icontrols.kakao.config.AsyncConfig;
import com.icontrols.kakao.entity.ParcelInfo;
import com.icontrols.kakao.entity.ThingsInfo;
import com.icontrols.kakao.entity.UserInfo;
import com.icontrols.kakao.repo.ParcelInfoRepository;
import com.icontrols.kakao.repo.ThingsInfoRepository;
import com.icontrols.kakao.socket.utils.IPGenerator;
import com.icontrols.kakao.socket.utils.SSLClient;
import com.icontrols.kakao.socket.utils.XMLConstants.REMOTE_KIND;
import com.icontrols.kakao.socket.utils.XMLGenerator;

@Service("kakaoMsgHandler")
public class KakaoMsgHandler {
	private final static Logger logger = LoggerFactory.getLogger(KakaoMsgHandler.class);
	private static final int 	REQ_TYPE_QUERY = 1;
	private static final int REQ_TYPE_UPDATE = 2;
	@Resource(name="asyncConfig")
	private AsyncConfig asyncConfig;
	
	@Resource(name="wallpadMsgSender")
	WallpadMsgSender wallpadMsgSender;
	
	@Autowired
	ParcelInfoRepository parcelRepo;
	
	/**
	 * 카카오 전용 인증 프로세스
	 * 월패드에 인증번호를 노출시키는 메시지를 전송한다.
	 * @param param
	 * 		
	 * @return
	 */
	public Map<String, Object> doAuthProcess(Map<String, Object> param){
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		try {
			int dong = Integer.parseInt((String)param.get("dong"));
			int ho = Integer.parseInt((String)param.get("ho"));
			String code = (String)param.get("code");
			
			String targetIP = IPGenerator.getIPFromDongHo(dong, ho);
			String twinid = "KAKAO"+dong+"_"+ho;
			String alias = "Kakao";
			StringBuffer sb = new StringBuffer();
			
			sb.append(XMLGenerator.MakeXMLHeaderForNotice(REMOTE_KIND.remote_register_new_device_auth_number.getContents()))
			.append("      <home_info dong=\"").append(dong).append("\" ho=\"").append(ho).append("\" ip=\"").append(targetIP).append("\" type=\"main\"/>\r\n")
			.append("      <device_info alias=\"").append(alias).append("\" twinid=\"").append(twinid).append("\" auth_number=\"" + code + "\" />\r\n")
			.append(XMLGenerator.MakeXMLTail());
			
			logger.info("[doAuthProcess]-[인증번호요청] - [" + dong + "동]-[" + ho + "호] - [" + code + " ]");
			
			String rcvMsg = SSLClient.sendToWallpad(targetIP, sb.toString());
			
			if(rcvMsg.trim().length() == 0) {
				retMap.put("result", ERROR_INFO.CODE.SUCCESS.getContents());
				retMap.put("resultCode", ERROR_INFO.CODE.SUCCESS.getmIdx());
			}
		}catch(Exception e) {
			retMap.put("result", ERROR_INFO.CODE.FAIL_999.getContents());
			retMap.put("resultCode", ERROR_INFO.CODE.FAIL_999.getmIdx());
			e.printStackTrace();
		}
		return retMap;
	}
	
	/**
	 * 기기 구독 / 해제 에 대한 메시지를 월패드에 전송한다. 
	 * 기기 구독이 돼 있지 않으면 이벤트를 받지 못한다.
	 * @param param
	 * @return
	 */
	public Map<String, Object> subscriptionProcess(Map<String, Object> param) {
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		try {
			Document doc = new WallpadMsgSender().remoteDeviceSubscription(param);
			
			if(doc == null) {
				logger.error("[subscriptionProcess ERROR] - Document null");
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				return retMap;
			}
			
			retMap.put("result", "success");
		}catch(Exception e) {
			logger.error("[subscriptionProcess ERROR] - " + e.getStackTrace());
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
		}
		return retMap;
	}
	
	public Map<String, Object> remoteRegisterNewDevice(Map<String, Object> param) { 
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		try {
			int dong = Integer.parseInt((String)param.get("dong"));
			int ho = Integer.parseInt((String)param.get("ho"));
			String code = (String)param.get("code");
			
			String targetIP = IPGenerator.getIPFromDongHo(dong, ho);
			String twinid = "KAKAO_"+dong+"_"+ho;
			String alias = "KAKAO HOME";
			StringBuffer sb = new StringBuffer();
			
			sb.append(XMLGenerator.MakeXMLHeaderForSUBS("remote_register_new_device"))
			.append("      <home_info dong=\"").append(dong).append("\" ho=\"").append(ho).append("\" ip=\"").append(targetIP).append("\" type=\"main\"/>\r\n")
			.append("      <device_info alias=\"").append(alias).append("\" twinid=\"").append(twinid).append("\"  />\r\n")
			.append(XMLGenerator.MakeXMLTail());
			
			logger.info("[remoteRegisterNewDevice] - [구독요청] - [" + dong + "동]-["+ho+"호]-["+twinid+"]");
			
			SSLClient.sendToWallpad(targetIP, sb.toString());
			
			retMap.put("result", ERROR_INFO.CODE.SUCCESS.getContents());
			retMap.put("resultCode", ERROR_INFO.CODE.SUCCESS.getmIdx());
		}catch(Exception e) {
			retMap.put("result", ERROR_INFO.CODE.FAIL_999.getContents());
			retMap.put("resultCode", ERROR_INFO.CODE.FAIL_999.getmIdx());
			e.printStackTrace();
		}
		return retMap;
	}
	
	public List<ThingsInfo> getConnectDevices(int dong, int ho){
		List<ThingsInfo> thingList = new ArrayList<>();
		
		try {
			Document doc = new WallpadMsgSender().remoteDeviceStatusList(dong, ho);
			
			if(doc == null) {
				logger.error("[ERROR]-[Document is Null] - [" + dong + "-" + ho + "]");
				return null;
			}
			
			XPath xpath = XPathFactory.newInstance().newXPath();
			Node iMap = (Node)xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
			String xmlResult = XmlUtil.getAttributeWithItem(iMap, "result");
			
			logger.debug("[KakaoMsgHandler - getConnectDevices] - xmlResult -> " + xmlResult);
			
			if(!xmlResult.equalsIgnoreCase("ok")) {
				logger.error("[KakaoMsgHandler - getConnectDevices] xmlResult not ok \r\n" + xmlResult);
				return null;
			}
			
			// 월패드와 연결된 상위 THINGS 갯수 확인
			Node dCount =  null;
			NodeList devList = (NodeList)xpath.evaluate("//imap/service/devinfo", doc, XPathConstants.NODESET);
			
			NodeUtil.asList(devList)
					.stream()
					.filter(pNode -> THINGS_INFO.isAvailableThings(XmlUtil.getAttributeWithItem(pNode, "name")))
					.forEach(pNode -> {
						String tmpModelId = XmlUtil.getAttributeWithItem(pNode, "name");
						int childCount = Integer.parseInt(XmlUtil.getAttributeWithItem(pNode, "value"));
						
						logger.debug("tmpModelID --> " + tmpModelId);
						try {
							if(tmpModelId.equalsIgnoreCase("light") || tmpModelId.equalsIgnoreCase("electric")) {
								for(int j=1; j <= childCount; j++) {
									String childName = j < 10 ? tmpModelId+"0"+j : tmpModelId+j;
									Node childNode = (Node)xpath.evaluate("//imap/service/devinfo/*[@name='"+ childName +"']", doc, XPathConstants.NODE);
									logger.debug("childName = " + childName);
									int tmpChildCount = Integer.parseInt(XmlUtil.getAttributeWithItem(childNode, "value"));
									
									NodeList childList = (NodeList)xpath.evaluate("//imap/service/devinfo/subdevinfo/*[@name='"+ childName +"']", doc, XPathConstants.NODESET);
									
									thingList.addAll(getChildThingsInfoList(dong, ho, tmpChildCount, childList));
								}
							}else {
								NodeList childList = (NodeList)xpath.evaluate("//imap/service/devinfo/*[@name='"+ tmpModelId +"']", doc, XPathConstants.NODESET);
								thingList.addAll(getChildThingsInfoList(dong, ho, childCount, childList));
							}
						}catch(Exception e) {
							logger.error("[ERROR] at stream - " + e.getMessage());
							e.printStackTrace();
						}
					});
			thingList.add(new ThingsInfo("elevator","ElevatorCaller", Integer.toString(dong), Integer.toString(ho)));
			thingList.add(new ThingsInfo("parcel","CommonParcel", Integer.toString(dong), Integer.toString(ho)));
			thingList.add(new ThingsInfo("parking","CommonParking", Integer.toString(dong), Integer.toString(ho)));
		}catch(Exception e) {
			logger.error("[ERROR] - " + e.getMessage());
			return null;
		}
		
		return thingList;
	}
	/**
	 * 월패드와 연결된 기기들의 전체 상태를 월패드에 요청하고 응답 받는다.
	 * 지원 기기가 추가되면 해당 부분은 수정이 필요하다. 
	 * 자식 노드로 subdevinfo가 있는 기기들을 잘 확인해야 한다.
	 * @param dong
	 * @param ho
	 * @return
	 */
	
	public Map<String, Object> getDevicesStatusList(int dong, int ho) {
		Map<String, Object> retMap = new HashMap<String, Object>();
		Map<String, String> metadata = new HashMap<String, String>();
		Map<String, Object> body = new HashMap<String, Object>();
		List<String> unreachableIds = new ArrayList<String>();
		
		try {
			Document doc = new WallpadMsgSender().remoteDeviceStatusList(dong, ho);
			
			if(doc == null) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[ERROR]-[Document is Null] - [" + dong + "-" + ho + "]");
				return retMap;
			}
			
			XPath xpath = XPathFactory.newInstance().newXPath();
			Node iMap = (Node)xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
			String xmlResult = XmlUtil.getAttributeWithItem(iMap, "result");
			
			logger.debug("[KakaoMsgHandler - getDeviceList] - xmlResult -> " + xmlResult);
			
			if(!xmlResult.equalsIgnoreCase("ok")) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[KakaoMsgHandler - getDeviceList] - [" + dong + "]-[" + ho + "] xmlResult not ok \r\n" + xmlResult);
				
				return retMap;
			}
			
			// 월패드와 연결된 상위 THINGS 갯수 확인
			Node dCount =  null;
			NodeList devList = (NodeList)xpath.evaluate("//imap/service/devinfo", doc, XPathConstants.NODESET);
			
			logger.debug("****device list***");
			logger.debug(devList.toString());
			
			NodeUtil.asList(devList)
					.stream()
					.filter(pNode -> THINGS_INFO.isAvailableThings(XmlUtil.getAttributeWithItem(pNode, "name")))
					.forEach(pNode -> {
						String tmpModelId = XmlUtil.getAttributeWithItem(pNode, "name");
						int childCount = Integer.parseInt(XmlUtil.getAttributeWithItem(pNode, "value"));
						
						List<Map<String, Object>> childNodeInfoList = null;
						logger.debug("[tmpModelID] --> " + tmpModelId + " [childCount] = " + childCount);
						try {
							// subdevinfo 를 가지고 있는 THINGS일 경우는 subdevinfo 들을 다시 구해야 한다.
							// 예를들어 light는 light01 ~ 05 등의 subdevinfo를 가진다.

							// livinglight와 light는 -> dimmer와 colorTemp를 조절이 가능한 조명이 포함될 수 있다.
							// 전등의 경우 월패드에서는 크게 livinglight 또는 light 로 나뉘고 속성은 같은반변 
							// 카카오에서는 light, LightDimmer, ColorLightDimmer 세가지로 나뉜다.
							// 월패드의 status는 (전원)/(디밍값)/(색상온도값) 으로 구성되며 
							// 전원을 제외하고 각 지원하지 않는 기능은 null로 전달된다. 
							// LightDimmer는 월패드에서 status가 on/40/null 로  status가 구성되고
							// ColorLightDimmer는 월패드에서 status가 on/40/10 의 형태 또는 status가 on/null/10 의형태로 구성된다. 
							// status의 구성요소에 맞게 capability가 구성되어야 한다. 
							
							
							if(tmpModelId.equalsIgnoreCase("light") || tmpModelId.equalsIgnoreCase("electric")) {
								for(int j=1; j <= childCount; j++) {
									String childName = j < 10 ? tmpModelId+"0"+j : tmpModelId+j;
									Node childNode = (Node)xpath.evaluate("//imap/service/devinfo/*[@name='"+ childName +"']", doc, XPathConstants.NODE);
									logger.debug("childName = " + childName + " // j = " + j);
									
									if(childNode == null) continue;
									
									String tmpValue = XmlUtil.getAttributeWithItem(childNode, "value");
									
									if(tmpValue == null) continue;
									
									int tmpchildCount = Integer.parseInt(tmpValue);
									
									NodeList childList = (NodeList)xpath.evaluate("//imap/service/devinfo/subdevinfo/*[@name='"+ childName +"']", doc, XPathConstants.NODESET);
									
									childNodeInfoList = getChildDeviceInfo(tmpchildCount, childList);
									logger.debug("child-->>" + childNodeInfoList.toString());
									if(childNodeInfoList != null)
										childNodeInfoList.forEach(tMap -> { body.putAll(tMap); });
								}
							}else {
								NodeList childList = (NodeList)xpath.evaluate("//imap/service/devinfo/*[@name='"+ tmpModelId +"']", doc, XPathConstants.NODESET);
								
								childNodeInfoList = getChildDeviceInfo(childCount, childList);
								
								logger.debug("child-->>" + childNodeInfoList.toString());
								
								if(childNodeInfoList != null)
									childNodeInfoList.forEach(tMap -> { body.putAll(tMap); });
							}
						}catch(Exception e) {
							retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
							logger.error("[ERROR] at stream - " + e.getMessage());
							e.printStackTrace();
						}
					});
			//aircon의 capa를 다시 조회한다.
			// aircon은 세대별로 지원하는 모드가 다를 수 있어서 월패드의 해당 세대의 에어컨 속성을 다시 조회해서 전달해줘야 한다.
			Map<String, Object> airconCapaMap = getAirconCapability(dong, ho);
			for(int i=1; i <=10; i++) {
				String _tmpEndpoint = "aircon_" + i;
				if(body.containsKey(_tmpEndpoint)) {
					Map<String, Object> _tmpDeviceMap = (HashMap)body.get(_tmpEndpoint);
					_tmpDeviceMap.put("capabilities", airconCapaMap.get("capabilities"));
				}
			}
			
			// elevator는 하드코딩.
			// elevator가 존재해야 카카오홈 앱에서 엘리베이터를 제어할 수 있다.
			Map<String, Object> eleMap = new HashMap<String, Object>();
			
			eleMap.put(KEY_INFO.BODY_KEY_DISPLAY_TYPE, THINGS_INFO.THINGS.ELEVATOR.getKakaoDisplayName());
			eleMap.putAll(getDeviceCapabilities(THINGS_INFO.THINGS.ELEVATOR.getThingsName(), null));
			eleMap.put(KEY_INFO.BODY_KEY_ENDPOINT_ID, THINGS_INFO.DEVICE_DETAIL.ELEVATOR.getModelId());
			eleMap.put(KEY_INFO.BODY_KEY_NAME, THINGS_INFO.THINGS.ELEVATOR.getDesc());
			eleMap.put(KEY_INFO.BODY_KEY_STATE, new HashMap<String, Object>());
			eleMap.put(KEY_INFO.BODY_KEY_ADDITIONALPROP, getAdditionalProp(KEY_INFO.BODY_KEY_ROOMPRESETNAME, THINGS_INFO.DEVICE_DETAIL.ELEVATOR.getRoomName()));
			body.put("elevator", eleMap);
			
			// 주차위치도 직접 입력해줘야 함. 
			Map<String, Object> parkingMap = new HashMap<String, Object>();
			parkingMap.put(KEY_INFO.BODY_KEY_DISPLAY_TYPE, THINGS_INFO.THINGS.PARKING.getKakaoDisplayName());
			parkingMap.putAll(getDeviceCapabilities(THINGS_INFO.THINGS.PARKING.getThingsName(), null));
			parkingMap.put(KEY_INFO.BODY_KEY_ENDPOINT_ID, THINGS_INFO.DEVICE_DETAIL.PARKING.getModelId());
			parkingMap.put(KEY_INFO.BODY_KEY_NAME, THINGS_INFO.THINGS.PARKING.getDesc());

			// 주차위치를 월패드에 물어봐서 입력해주어야함.
			parkingMap.put(KEY_INFO.BODY_KEY_STATE, getParkingList(dong, ho));
			parkingMap.put(KEY_INFO.BODY_KEY_ADDITIONALPROP, getAdditionalProp(KEY_INFO.BODY_KEY_ROOMPRESETNAME, THINGS_INFO.DEVICE_DETAIL.PARKING.getRoomName()));
			body.put("parking", parkingMap);
			
			// 택배 정보를 조회해서 알려주어야 함.
			Map<String, Object> parcelMap = getParcelList(dong, ho);
			
			if(parcelMap != null)
				body.put("parcel", parcelMap);
			
			if(SERVERINFO.KAKAO_I_WALLPAD_APPLY) {
				Map<String, Object> wpMap = new HashMap<String, Object>();
				String wpEndpointId = "WALLPAD_"+SERVERINFO.danzi_code+"_"+dong+"_"+ho;
				wpMap.put("endpointId", wpEndpointId);
				wpMap.put("name", "월패드");
				wpMap.put("displayType", "DummyWallPad");
				wpMap.put("capabilities", new ArrayList<>());
				wpMap.put("state", "{}");
				wpMap.put("additionalProperties", null);
				
				body.put(wpEndpointId, wpMap);
			}
			
		}catch(Exception e) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
			logger.error("[ERROR] - " + e.getMessage());
		}
		
		metadata.put("aptCode", SERVERINFO.danzi_code);
		retMap.put(KEY_INFO.HTTP_KEY_METADATA, metadata); 
		retMap.put(KEY_INFO.HTTP_KEY_UNREACHABLEIDS, unreachableIds);
		retMap.put(KEY_INFO.HTTP_KEY_BODY, body);
		
		return retMap;
	}
	
	public Map<String, Object> getDevicesStatusListTemp(int dong, int ho) {
		Map<String, Object> retMap = new HashMap<String, Object>();
		Map<String, String> metadata = new HashMap<String, String>();
		Map<String, Object> body = new HashMap<String, Object>();
		List<String> unreachableIds = new ArrayList<String>();
		
		try {
			Document doc = new WallpadMsgSender().remoteDeviceStatusList(dong, ho);
			
			if(doc == null) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[ERROR]-[Document is Null] - [" + dong + "-" + ho + "]");
				return retMap;
			}
			
			XPath xpath = XPathFactory.newInstance().newXPath();
			Node iMap = (Node)xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
			String xmlResult = XmlUtil.getAttributeWithItem(iMap, "result");
			
			logger.debug("[KakaoMsgHandler - getDeviceList] - xmlResult -> " + xmlResult);
			
			if(!xmlResult.equalsIgnoreCase("ok")) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[KakaoMsgHandler - getDeviceList] - [" + dong + "]-[" + ho + "] xmlResult not ok \r\n" + xmlResult);
				
				return retMap;
			}
			
			// 월패드와 연결된 상위 THINGS 갯수 확인
			Node dCount =  null;
			NodeList devList = (NodeList)xpath.evaluate("//imap/service/devinfo", doc, XPathConstants.NODESET);
			
			logger.debug("****device list***");
			logger.debug(devList.toString());
			
			NodeUtil.asList(devList)
					.stream()
					.filter(pNode -> THINGS_INFO.isAvailableThings(XmlUtil.getAttributeWithItem(pNode, "name")))
					.forEach(pNode -> {
						String tmpModelId = XmlUtil.getAttributeWithItem(pNode, "name");
						int childCount = Integer.parseInt(XmlUtil.getAttributeWithItem(pNode, "value"));
						
						List<Map<String, Object>> childNodeInfoList = null;
						logger.debug("[tmpModelID] --> " + tmpModelId + " [childCount] = " + childCount);
						try {
							// subdevinfo 를 가지고 있는 THINGS일 경우는 subdevinfo 들을 다시 구해야 한다.
							// 예를들어 light는 light01 ~ 05 등의 subdevinfo를 가진다.

							// livinglight와 light는 -> dimmer와 colorTemp를 조절이 가능한 조명이 포함될 수 있다.
							// 전등의 경우 월패드에서는 크게 livinglight 또는 light 로 나뉘고 속성은 같은반변 
							// 카카오에서는 light, LightDimmer, ColorLightDimmer 세가지로 나뉜다.
							// 월패드의 status는 (전원)/(디밍값)/(색상온도값) 으로 구성되며 
							// 전원을 제외하고 각 지원하지 않는 기능은 null로 전달된다. 
							// LightDimmer는 월패드에서 status가 on/40/null 로  status가 구성되고
							// ColorLightDimmer는 월패드에서 status가 on/40/10 의 형태 또는 status가 on/null/10 의형태로 구성된다. 
							// status의 구성요소에 맞게 capability가 구성되어야 한다. 
							
							
							if(tmpModelId.equalsIgnoreCase("light") || tmpModelId.equalsIgnoreCase("electric")) {
								for(int j=1; j <= childCount; j++) {
									String childName = j < 10 ? tmpModelId+"0"+j : tmpModelId+j;
									Node childNode = (Node)xpath.evaluate("//imap/service/devinfo/*[@name='"+ childName +"']", doc, XPathConstants.NODE);
									logger.debug("childName = " + childName + " // j = " + j);
									
									if(childNode == null) continue;
									
									String tmpValue = XmlUtil.getAttributeWithItem(childNode, "value");
									
									if(tmpValue == null) continue;
									
									int tmpchildCount = Integer.parseInt(tmpValue);
									
									NodeList childList = (NodeList)xpath.evaluate("//imap/service/devinfo/subdevinfo/*[@name='"+ childName +"']", doc, XPathConstants.NODESET);
									
									childNodeInfoList = getChildDeviceInfo(tmpchildCount, childList);
									logger.debug("child-->>" + childNodeInfoList.toString());
									if(childNodeInfoList != null)
										childNodeInfoList.forEach(tMap -> { body.putAll(tMap); });
								}
							}else {
								NodeList childList = (NodeList)xpath.evaluate("//imap/service/devinfo/*[@name='"+ tmpModelId +"']", doc, XPathConstants.NODESET);
								
								childNodeInfoList = getChildDeviceInfo(childCount, childList);
								
								logger.debug("child-->>" + childNodeInfoList.toString());
								
								if(childNodeInfoList != null)
									childNodeInfoList.forEach(tMap -> { body.putAll(tMap); });
							}
						}catch(Exception e) {
							retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
							logger.error("[ERROR] at stream - " + e.getMessage());
							e.printStackTrace();
						}
					});
			
			Map<String, Object> airconCapaMap = getAirconCapability(dong, ho);
			for(int i=1; i <=10; i++) {
				String _tmpEndpoint = "aircon_" + i;
				if(body.containsKey(_tmpEndpoint)) {
					Map<String, Object> _tmpDeviceMap = (HashMap)body.get(_tmpEndpoint);
					_tmpDeviceMap.put("capabilities", airconCapaMap.get("capabilities"));
				}
			}
			
			
			// elevator는 하드코딩.
			// elevator가 존재해야 카카오홈 앱에서 엘리베이터를 제어할 수 있다.
			Map<String, Object> eleMap = new HashMap<String, Object>();
			
			eleMap.put(KEY_INFO.BODY_KEY_DISPLAY_TYPE, THINGS_INFO.THINGS.ELEVATOR.getKakaoDisplayName());
			eleMap.putAll(getDeviceCapabilities(THINGS_INFO.THINGS.ELEVATOR.getThingsName(), null));
			eleMap.put(KEY_INFO.BODY_KEY_ENDPOINT_ID, THINGS_INFO.DEVICE_DETAIL.ELEVATOR.getModelId());
			eleMap.put(KEY_INFO.BODY_KEY_NAME, THINGS_INFO.THINGS.ELEVATOR.getDesc());
			eleMap.put(KEY_INFO.BODY_KEY_STATE, new HashMap<String, Object>());
			eleMap.put(KEY_INFO.BODY_KEY_ADDITIONALPROP, getAdditionalProp(KEY_INFO.BODY_KEY_ROOMPRESETNAME, THINGS_INFO.DEVICE_DETAIL.ELEVATOR.getRoomName()));
			body.put("elevator", eleMap);
			
			// 주차위치도 직접 입력해줘야 함. 
			Map<String, Object> parkingMap = new HashMap<String, Object>();
			parkingMap.put(KEY_INFO.BODY_KEY_DISPLAY_TYPE, THINGS_INFO.THINGS.PARKING.getKakaoDisplayName());
			parkingMap.putAll(getDeviceCapabilities(THINGS_INFO.THINGS.PARKING.getThingsName(), null));
			parkingMap.put(KEY_INFO.BODY_KEY_ENDPOINT_ID, THINGS_INFO.DEVICE_DETAIL.PARKING.getModelId());
			parkingMap.put(KEY_INFO.BODY_KEY_NAME, THINGS_INFO.THINGS.PARKING.getDesc());

			// 주차위치를 월패드에 물어봐서 입력해주어야함.
			parkingMap.put(KEY_INFO.BODY_KEY_STATE, getParkingList(dong, ho));
			parkingMap.put(KEY_INFO.BODY_KEY_ADDITIONALPROP, getAdditionalProp(KEY_INFO.BODY_KEY_ROOMPRESETNAME, THINGS_INFO.DEVICE_DETAIL.PARKING.getRoomName()));
			body.put("parking", parkingMap);
			
			// 택배 정보를 조회해서 알려주어야 함.
			Map<String, Object> parcelMap = getParcelList(dong, ho);
			
			if(parcelMap != null)
				body.put("parcel", parcelMap);
			
//			if(SERVERINFO.KAKAO_I_WALLPAD_APPLY) {
//				Map<String, Object> wpMap = new HashMap<String, Object>();
//				String wpEndpointId = "WALLPAD_"+SERVERINFO.danzi_code+"_"+dong+"_"+ho;
//				wpMap.put("endpointId", wpEndpointId);
//				wpMap.put("name", "월패드");
//				wpMap.put("displayType", "DummyWallPad");
//				wpMap.put("capabilities", new ArrayList<>());
//				wpMap.put("state", "{}");
//				wpMap.put("additionalProperties", null);
//				
//				body.put(wpEndpointId, wpMap);
//			}
			
		}catch(Exception e) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
			logger.error("[ERROR] - " + e.getMessage());
		}
		
		metadata.put("aptCode", SERVERINFO.danzi_code);
		retMap.put(KEY_INFO.HTTP_KEY_METADATA, metadata); 
		retMap.put(KEY_INFO.HTTP_KEY_UNREACHABLEIDS, unreachableIds);
		retMap.put(KEY_INFO.HTTP_KEY_BODY, body);
		
		return retMap;
	}
	
	
	private Map<String, Object>getParkingList(int dong, int ho){
		Map<String, Object> retMap = new HashMap<String, Object>();
		try {
			Document doc = new WallpadMsgSender().remoteGetParkingList(dong, ho);
			if(doc == null) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[ERROR]-[getParkingList]-[Document is Null] - [" + dong + "-" + ho + "]");
				return retMap;
			}
			
			XPath xpath = XPathFactory.newInstance().newXPath();
			Node iMap = (Node)xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
			String xmlResult = XmlUtil.getAttributeWithItem(iMap, "result");
			
			logger.debug("[KakaoMsgHandler - getParkingList] - xmlResult -> " + xmlResult);
			
			if(!xmlResult.equalsIgnoreCase("ok")) {
//				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[KakaoMsgHandler - getParkingList] - [" + dong + "]-[" + ho + "] xmlResult not ok \r\n" + xmlResult);
				retMap.put("parkingLocations", new ArrayList<>());
				return retMap;
			}
			Node parkingList = (Node)xpath.evaluate("//imap/service/parking_list",doc, XPathConstants.NODE);
			int parkingListCnt = Integer.parseInt(XmlUtil.getAttributeWithItem(parkingList, "total_count"));
			
			logger.debug("[KakaoMsgHandler - getParkingList] - 1 parkingListCnt = " + parkingListCnt);
			
			if(parkingListCnt == 0) return retMap;
			
			List<Map<String,Object>> parkingLocations = new ArrayList<>();
			NodeUtil.asList((NodeList)xpath.evaluate("//imap/service/parking_list/parking_info", doc, XPathConstants.NODESET))
					.stream()
					.forEach(pNode ->{
						String carNum = XmlUtil.getAttributeWithItem(pNode, "car_num");
						String location = XmlUtil.getAttributeWithItem(pNode, "location");
						Map<String, Object> _parkingInfo = new HashMap<String, Object>();
						
						_parkingInfo.put("carno", carNum);
						_parkingInfo.put("name", location);
						_parkingInfo.put("parked", true);
						
						parkingLocations.add(_parkingInfo);
					});
			retMap.put("parkingLocations", parkingLocations);
		}catch(Exception e) {
			logger.error("[ERROR]-[getParkingList]-" + e.getMessage());
		}
		
		logger.info("[parkingList]-[ret]-" + retMap.toString());
		
		return retMap;
	}
	
	private Map<String, Object>getParcelList(int dong, int ho){
		Map<String, Object> retMap = new HashMap<String, Object>();
		try {
			List<ParcelInfo> list = parcelRepo.findByDonghoId(dong+"/"+ho);
			// 해당 세대의 조회된 택배DB가 없을 경우 DB를 생성한다.
			ParcelInfo parcelObj = null;
			if(list.size() == 0) {
				parcelObj = new ParcelInfo(dong+"/"+ho, 0, null, null);
				parcelRepo.save(parcelObj);
			}else {
				parcelObj = list.get(0);
			}
			Map<String, Object> stateMap = new HashMap<>();
			List<Object> parcelList = new ArrayList<>();
			logger.info("[" + dong + "/" + ho +"]-[parcel]-" + parcelObj.toString());
			
			for(int i=0; i < parcelObj.getQuantity(); i++) {
				Map<String, Object> _obj = new HashMap<>();
				_obj.put("name", "택배 보관함");
				_obj.put("lastDeliveredTime", parcelObj.getLstUptTime());
				parcelList.add(_obj);
			}
			
			stateMap.put("parcelLocations", parcelList);
			
			retMap.put(KEY_INFO.BODY_KEY_DISPLAY_TYPE, THINGS_INFO.THINGS.PARCEL.getKakaoDisplayName());
			retMap.putAll(getDeviceCapabilities(THINGS_INFO.THINGS.PARCEL.getThingsName(), null));
			retMap.put(KEY_INFO.BODY_KEY_ENDPOINT_ID, THINGS_INFO.DEVICE_DETAIL.PARCEL.getModelId());
			retMap.put(KEY_INFO.BODY_KEY_NAME, THINGS_INFO.THINGS.PARCEL.getDesc());
			retMap.put(KEY_INFO.BODY_KEY_STATE, stateMap);
			retMap.put(KEY_INFO.BODY_KEY_ADDITIONALPROP, getAdditionalProp(KEY_INFO.BODY_KEY_ROOMPRESETNAME, THINGS_INFO.DEVICE_DETAIL.PARKING.getRoomName()));
		}catch(Exception e) {
			logger.error("[ERROR]-[getParcelList]-" + e.getMessage());
			e.printStackTrace();
			return null;
		}
		return retMap;
	}
	
	public Map<String, Object>getAirconCapability(int dong, int ho){
		String jsonString = THINGS_INFO.THINGS_CAPABILITY.AIRCON.getCapJsonStr();
		
		Map<String, Object> capabilities = CommonUtil.jsonToMap(jsonString);
		try {
			Document doc = new WallpadMsgSender().remoteGetAirconAttributes(dong, ho);
			if(doc == null) {
				logger.error("[ERROR]-[getAirconCapability]-[Document is Null] - [" + dong + "-" + ho + "]");
				return capabilities;
			}
			
			XPath xpath = XPathFactory.newInstance().newXPath();
			Node iMap = (Node)xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
			String xmlResult = XmlUtil.getAttributeWithItem(iMap, "result");
			
			logger.debug("[KakaoMsgHandler - 	private] - xmlResult -> " + xmlResult);
			
			if(!xmlResult.equalsIgnoreCase("ok")) {
//				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[KakaoMsgHandler - getAirconCapability] - [" + dong + "]-[" + ho + "] xmlResult not ok \r\n" + xmlResult);
				return capabilities;
			}
			
			Node attributes = (Node)xpath.evaluate("//imap/service/attributes",doc, XPathConstants.NODE);
			int totalAirconCount = Integer.parseInt(XmlUtil.getAttributeWithItem(attributes, "total_count"));
			
			logger.debug("[KakaoMsgHandler - getAirconCapability] - totalAirconCount = " + totalAirconCount);
			
			Node coolTempAttributes = (Node)xpath.evaluate("//imap/service/attributes/cool_temperature",doc, XPathConstants.NODE);
			int minCoolTemperature = Integer.parseInt(XmlUtil.getAttributeWithItem(coolTempAttributes, "min"));
			int maxCoolTemperature = Integer.parseInt(XmlUtil.getAttributeWithItem(coolTempAttributes, "max"));
			
			Node heatTempAttributes = (Node)xpath.evaluate("//imap/service/attributes/heat_temperature",doc, XPathConstants.NODE);
			int minHeatTemperature = Integer.parseInt(XmlUtil.getAttributeWithItem(heatTempAttributes, "min"));
			int maxHeatTemperature = Integer.parseInt(XmlUtil.getAttributeWithItem(heatTempAttributes, "max"));
			
			int minTemperature = minCoolTemperature<=minHeatTemperature?minCoolTemperature:minHeatTemperature;
			int maxTemperature = maxCoolTemperature>=maxHeatTemperature?maxCoolTemperature:maxHeatTemperature;
			
			logger.debug("[KakaoMsgHandler - getAirconCapability] - minTemperature = " + minTemperature + " - maxTemperature =" + maxTemperature);
			
			Node modeAttributes = (Node)xpath.evaluate("//imap/service/attributes/mode",doc, XPathConstants.NODE);
			String strModeOption = XmlUtil.getElemetTextValue(modeAttributes);
			String[] arrModeOption = strModeOption.split("/");
			logger.debug("[KakaoMsgHandler - getAirconCapability] - arrModeOption = " + arrModeOption);
			
			Node airFlowAttributes = (Node)xpath.evaluate("//imap/service/attributes/air_flow",doc, XPathConstants.NODE);
			String strAirFlowOption = XmlUtil.getElemetTextValue(airFlowAttributes);
			String[] arrAirFlowOption = strAirFlowOption.split("/");
			logger.debug("[KakaoMsgHandler - getAirconCapability] - arrAirFlowOption = " + arrAirFlowOption);
			
			List<HashMap<String, Object>> capabilityList = (ArrayList)capabilities.get("capabilities");
			logger.debug("[KakaoMsgHandler - getAirconCapability] - capabilityList = " + capabilityList.toString());
			
			for(HashMap<String, Object> capa:capabilityList) {
				String capaName = (String) capa.get("name");
				logger.debug("[KakaoMsgHandler - getAirconCapability] - capaName = " + capaName);
				Map<String, Object> attr = (HashMap)capa.get("attributes");
				logger.debug("[KakaoMsgHandler - getAirconCapability] - attr = " + attr.toString());
				if(capaName.equalsIgnoreCase("AirConditioner")) {
					Map<String, Object> modeCapa = (HashMap)attr.get("mode");
					logger.debug("[KakaoMsgHandler - getAirconCapability] - modeCapa = " + modeCapa.toString());
					
					List<String> modeAttrList = new ArrayList();
					for(String mode:arrModeOption) {
						logger.debug("[KakaoMsgHandler - getAirconCapability] - modeCapa-detail = " + mode);
						if(mode.equalsIgnoreCase("auto"))
							modeAttrList.add("auto");
						else if(mode.equalsIgnoreCase("cool"))
							modeAttrList.add("cool");
						else if(mode.equalsIgnoreCase("heat"))
							modeAttrList.add("heat");
						else if(mode.equalsIgnoreCase("fan"))
							modeAttrList.add("fan_only");
						else if(mode.equalsIgnoreCase("dehum"))
							modeAttrList.add("dehumidify");
					}
					modeCapa.clear();
					modeCapa.put("writable", modeAttrList);
					modeCapa.put("supported", modeAttrList);
					modeCapa.put("mutability", "rw");
				}else if(capaName.equalsIgnoreCase("AirFlow")) {
					Map<String, Object> airFlowCapa = (HashMap)attr.get("fanSpeed");
					List<String> airFlowAttrList = new ArrayList();
					for(String mode:arrAirFlowOption) {
						if(mode.equalsIgnoreCase("auto"))
							airFlowAttrList.add("auto");
						else if(mode.equalsIgnoreCase("low"))
							airFlowAttrList.add("low");
						else if(mode.equalsIgnoreCase("mid"))
							airFlowAttrList.add("middle");
						else if(mode.equalsIgnoreCase("high"))
							airFlowAttrList.add("high");
					}
					airFlowCapa.clear();
					airFlowCapa.put("writable", airFlowAttrList);
					airFlowCapa.put("supported", airFlowAttrList);
					airFlowCapa.put("mutability", "rw");
				}else if(capaName.equalsIgnoreCase("Temperature")) {
					Map<String, Object> tempAttr = new HashMap<>();
					tempAttr.put("mutability", "r");
					tempAttr.put("max", maxTemperature);
					tempAttr.put("min", minTemperature);
					tempAttr.put("mutability", "r");
					attr.clear();
					attr.put("currentTemperature", tempAttr);
					
				}else if(capaName.equalsIgnoreCase("TemperatureSetpoint")) {
					Map<String, Object> tempAttr = new HashMap<>();
					tempAttr.put("mutability", "r");
					tempAttr.put("max", maxTemperature);
					tempAttr.put("min", minTemperature);
					tempAttr.put("mutability", "r");
					attr.clear();
					attr.put("targetSetpoint", tempAttr);
				}
			}
		}catch(Exception e) {
			logger.error("[ERROR]-[getAirconCapability]-[Exception]-" + e.getMessage());
		}
		
		logger.info("[getAirconCapability]-" + capabilities.toString());
		return capabilities;
	}
	
	public static Map<String, Object>getAirconEnableTemperature(int dong, int ho, String endpointId){
		// 디바이스의 현재 상태를 먼저 조회한다. 
		// 디바이스의 모드를 먼저 확인한다.
		// 디바이스의 설정 온도 가능 범위를 확인한다. 
		// 디바이스가 냉방이면 냉방의 가능범위를 응답한다.
		// 디바이스가 난방이면 난방의 가능범위를 응답한다.
		// 디바이스가 그외 모드이면 냉방과 난방의 교집합으로 응답한다.
		Map<String, Object> retMap = new HashMap<String, Object>();
		try {
			Map<String, Object> deviceStateMap= getDeviceStatus(dong, ho, endpointId);
			Map<String, Object> statusMap = (HashMap)deviceStateMap.get(endpointId);
			if(statusMap.containsKey("mode")) {
				String mode = (String)statusMap.get("mode");
				
				Document doc = new WallpadMsgSender().remoteGetAirconAttributes(dong, ho);
				if(doc == null) {
					logger.error("[ERROR]-[getAirconEnableTemperature]-[Document is Null] - [" + dong + "-" + ho + "]");
					return null;
				}
				
				XPath xpath = XPathFactory.newInstance().newXPath();
				Node iMap = (Node)xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
				String xmlResult = XmlUtil.getAttributeWithItem(iMap, "result");
				
				logger.debug("[KakaoMsgHandler - 	private] - xmlResult -> " + xmlResult);
				
				if(!xmlResult.equalsIgnoreCase("ok")) {
//					retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
					logger.error("[KakaoMsgHandler - getAirconEnableTemperature] - [" + dong + "]-[" + ho + "] xmlResult not ok \r\n" + xmlResult);
					return null;
				}
				
				Node coolTempAttributes = (Node)xpath.evaluate("//imap/service/attributes/cool_temperature",doc, XPathConstants.NODE);
				int minCoolTemperature = Integer.parseInt(XmlUtil.getAttributeWithItem(coolTempAttributes, "min"));
				int maxCoolTemperature = Integer.parseInt(XmlUtil.getAttributeWithItem(coolTempAttributes, "max"));
				
				Node heatTempAttributes = (Node)xpath.evaluate("//imap/service/attributes/heat_temperature",doc, XPathConstants.NODE);
				int minHeatTemperature = Integer.parseInt(XmlUtil.getAttributeWithItem(heatTempAttributes, "min"));
				int maxHeatTemperature = Integer.parseInt(XmlUtil.getAttributeWithItem(heatTempAttributes, "max"));
				
				
				if(mode.equalsIgnoreCase("cool")) {
					retMap.put("min", minCoolTemperature);
					retMap.put("max", maxCoolTemperature);
				}else if(mode.equalsIgnoreCase("heat")) {
					retMap.put("min", minHeatTemperature);
					retMap.put("max", maxHeatTemperature);
				}else {
					retMap.put("min", minCoolTemperature >= minHeatTemperature?minCoolTemperature:minHeatTemperature);
					retMap.put("max", maxCoolTemperature <= maxHeatTemperature?maxCoolTemperature:maxHeatTemperature);
				}
			}else {
				/// 에러처리..
				logger.error("[ERROR]-[getAirconEnableTemperature]-["+dong+"]-["+ho+"]-["+endpointId+"]-" + deviceStateMap.toString());
				return null;
			}
		}catch(Exception e) {
			logger.error("[ERROR]-[getAirconEnableTemperature]-[Exception]-" + e.getMessage());
			return null;
		}
		return retMap;
	}
	
	
	/**
	 * 동 호 정보 EndpointID 로 월패드에 상태 조회 메세지를 전송하고 결과를 반환 받는다. 
	 * @param dong
	 * @param ho
	 * @param endpointId
	 * @return
	 */
	public static Map<String, Object> getDeviceStatus(int dong, int ho, String endpointId){
		Map<String, Object> retMap = new HashMap<String, Object>();
		String modelId = "";
		int devNum = 1;

		Document doc = null;

		// endpoint Id로 존재하는 THINGS 인지 확인 
		// 발생 ERROR FAIL_NONEXIST
		try {
			String[] tmpSplit = endpointId.split("_");
			
			modelId = tmpSplit[0];
			devNum = Integer.parseInt(tmpSplit[1]);	// 회로 번호
			
			if(!THINGS_INFO.isExistThings(modelId)) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_NONEXIST));
				logger.error("[ERROR]-[(" + endpointId + ") - Non Exist]");
				return retMap;
			}
		}catch(Exception e) { 
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_NONEXIST));
			logger.error("[ERROR]-[(" + endpointId + ") - Non Exist]");
			return retMap;
		}
		
		try {
			doc = new WallpadMsgSender().remoteDeviceStatus(dong, ho, modelId, devNum);
			
			if(doc == null) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[ERROR]-[Document is Null] - [" + dong + "-" + ho + "] ["+ modelId +"]");
				return retMap;
			}
		}catch(Exception e) { 
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_NONEXIST));
			logger.error("[ERROR]-[(" + endpointId + ") - Non Exist]");
			return retMap;
		}
		
		return getStatusMap(endpointId, doc, REQ_TYPE_QUERY, dong, ho);
	}
	
	/**
	 * 월패드에 장치 제어 명령을 보내고 결과를 리턴한다.
	 * @param dong
	 * @param ho
	 * @param endpointId
	 * @param actionName
	 * @param desired
	 * @return
	 */
	public Map<String, Object> sendDeviceAction(int dong, int ho, String endpointId, String actionName, Map<String, Object> desired){
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		String[] tmpSplit = endpointId.split("_");
		String modelId = (tmpSplit[0].equals("dimmer") || tmpSplit[0].equals("colorDimmer"))?tmpSplit[1]:tmpSplit[0];
		
		//엘리베이터 특정. -> 엘리베이터는 tmpSplit length가 1이다.
		// 모드 endpoint id는 _ <- 언더바가 포함돼 있지만 엘리베이터는 그렇지 않음
		// 엘리베이터가 아닌데도 tmpSplit length가 1인경우는 존재하지 않는 디바이스로 간주.
		if(tmpSplit.length == 1 && !modelId.equals(THINGS_INFO.DEVICE_DETAIL.ELEVATOR.getModelId())) {
			logger.error("[sendDeviceAction ERROR] - this is NonExist => " + modelId);
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_NONEXIST));
			
			return retMap;
		}
		
		int devNum = 1;
		
		if(tmpSplit[0].equals("dimmer") || tmpSplit[0].equals("colorDimmer")) devNum = Integer.parseInt(tmpSplit[2]);
		else if(modelId.equals(THINGS_INFO.DEVICE_DETAIL.ELEVATOR.getModelId())) devNum = 1;
		else devNum = Integer.parseInt(tmpSplit[1]);
		THINGS_INFO.THINGS things = THINGS_INFO.getThingsType(endpointId);
		
		if(things == null) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_REQ));
			logger.error("[ERROR] - things null ==> INVALID REQ");
			return retMap;
		}
		
		String action = KakaoMsgHandler.setDeviceActionStr(dong, ho, endpointId, modelId, actionName, desired);
		
		if(action.length() == 0) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_REQ));
			logger.error("[ERROR] - actions length --> 0 IVALID REQ " );
			return retMap;
		}
		
		Document doc = null;
		for(int i=0; i < 3; i++) {
			doc = new WallpadMsgSender().remoteDeviceControl(
																dong, 
																ho, 
																modelId, 
																devNum, 
																action
															);
			if(doc != null) break;
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		if(doc == null) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
			return retMap;
		}
		
		return getStatusMap(endpointId, doc, REQ_TYPE_UPDATE, dong, ho);
	}
	
	/**
	 * 상태조회 요청 메세지에 대한 응답 메세지를 파싱하여 카카오 형식으로 변환한다.
	 * @param endpointId
	 * @param doc
	 * @return
	 */
	public static Map<String, Object> getStatusMap(String endpointId, Document doc, int req_type, int dong, int ho){
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		XPath xpath = XPathFactory.newInstance().newXPath();
		String status = "";
		try {
			Node iMap = (Node) xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
			String xmlResult = XmlUtil.getAttributeWithItem(iMap, "result");
			
			if(!xmlResult.equalsIgnoreCase("ok")) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_REQ)); 
				logger.error("[ERROR]-[xmlResult is not ok \r\n" + xmlResult);
				return retMap;
			}
			
			status = (String) xpath.evaluate("//imap/service/status", doc, XPathConstants.STRING);
			
			Map<String, Object> state = setDeviceState(endpointId, status);
			retMap.put(endpointId, state);
			
		
		} catch (XPathExpressionException e) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
			e.printStackTrace();
		}
		
		if(req_type == REQ_TYPE_UPDATE) {
			try {
				String logMessage = "";
				if(endpointId.equalsIgnoreCase("elevator")) {
					logMessage = "[KAKAO Device Result] ip: " + SERVERINFO.danziServerExIp + " dong: " + dong + "  ho: " + ho + " model_id: elevator dev_num: 1 status: down";
				}else {
					// 기기정보를 알아와요
					String arrEndpointId[]  = endpointId.split("_");
					
					String str_modelId = arrEndpointId[arrEndpointId.length-2]; 
					String str_devNum = arrEndpointId[arrEndpointId.length-1]; 
					
					logMessage = "[KAKAO Device Result] ip: "+ SERVERINFO.danziServerExIp +" dong: "+ dong + "ho: " + ho + " model_id: " + str_modelId + " dev_num: " + str_devNum + " status: " + status;
				}
				logger.info(logMessage);
			}catch(Exception e) {
				logger.error("[ELK Log Parsing Error] - " + endpointId + " status = " + status + "\r\n" + e.getMessage());
			}
		}
		
		logger.debug("[UPDATE RESULT] - " + retMap.toString());
		return retMap;
	}
	
	
	/**
	 * 이벤트 발생시 카카오에 전송하는 Body를 반환한다.
	 * @param dong
	 * @param ho
	 * @param body
	 * @return
	 * @throws Exception
	 */
	public Map<String, Object> setKakaoEventMsg(UserInfo userInfo, Map<String, Object> body) throws Exception{
		Map<String, Object> retMap = new HashMap<String, Object>();
		List<String> unreachableIds = new ArrayList<>();
		
		retMap.put(KEY_INFO.HTTP_KEY_ID, UUID.randomUUID().toString()); 
		retMap.put(KEY_INFO.HTTP_KEY_TYPE, TYPE.REQ_TYPE.REPORTSTATE.getTypeName());
		retMap.put(KEY_INFO.HTTP_KEY_AUTHORIZATION, userInfo.getKakaoAccessToken());
		retMap.put(KEY_INFO.HTTP_KEY_VERSION, "1");
		retMap.put(KEY_INFO.HTTP_KEY_UNREACHABLEIDS, unreachableIds);
		retMap.put(KEY_INFO.HTTP_KEY_BODY, body);
		
		return retMap;
	}
	
	public Map<String, Object> setKDeleteIdsMsg(String dong, String ho, UserInfo userInfo){
		ThingsInfoRepository tRepo = (ThingsInfoRepository)BeanUtils.getBean("thingsInfoRepository");
		
	
		List<ThingsInfo> tList = tRepo.findByDongAndHo(dong, ho);
		
		List<String> endpointIds = new ArrayList<String>();
		
		for(ThingsInfo tInfo:tList) endpointIds.add(tInfo.getEndpointId());
		
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		retMap.put(KEY_INFO.HTTP_KEY_ID, UUID.randomUUID().toString()); 
		retMap.put(KEY_INFO.HTTP_KEY_TYPE, TYPE.REQ_TYPE.REPORTSTATE.getTypeName());
		retMap.put(KEY_INFO.HTTP_KEY_AUTHORIZATION, userInfo.getKakaoAccessToken());
		retMap.put(KEY_INFO.HTTP_KEY_VERSION, "1");
		retMap.put(KEY_INFO.HTTP_KEY_DELETEDIDS, endpointIds);
		
		return retMap;
	}
	/**
	 * 지원하는 기기들의 Capability 를 카카오 형식에 맞게 추출한다.
	 * 기기가 추가되게 되면 목록을 수정해야한다.
	 * @param modelId
	 * @return
	 */
	public static Map<String, Object> getDeviceCapabilities(String modelId, String status){
		THINGS_INFO.THINGS things = THINGS_INFO.getThingsType(modelId);
		
		String jsonString = "";
		
		switch(things.getEndPoint()) {
		case THINGS_INFO.THINGS_LIGHT:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.LIGHT.getCapJsonStr();
			break;
		case THINGS_INFO.THINGS_LIGHT_DIMMER:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.LIGHT_DIMMER.getCapJsonStr();
			break;
			
		case THINGS_INFO.THINGS_COLOR_LIGHT_DIMMER:
			String[] splitStr = status.split("/");
			if(splitStr[1].equals("null"))
				jsonString = THINGS_INFO.THINGS_CAPABILITY.COLOR_LIGHT.getCapJsonStr();
			else
				jsonString = THINGS_INFO.THINGS_CAPABILITY.COLOR_LIGHT_DIMMER.getCapJsonStr();
			break;
			
		case THINGS_INFO.THINGS_TEMPER:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.TEMPER.getCapJsonStr();
			break;
			
		case THINGS_INFO.THINGS_GAS:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.GAS.getCapJsonStr();
			break;
			
		case THINGS_INFO.THINGS_VENTIL:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.VENTIL.getCapJsonStr();
			break;
			
		case THINGS_INFO.THINGS_ELEVATOR:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.ELEVATOR.getCapJsonStr();
			break;
			
		case THINGS_INFO.THINGS_ELECPLUG:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.ELECTRIC.getCapJsonStr();
			break;
		case THINGS_INFO.THINGS_PARKING:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.PARKING.getCapJsonStr();
			break;
		case THINGS_INFO.THINGS_PARCEL:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.PARCEL.getCapJsonStr();
			break;
		case THINGS_INFO.THINGS_AIR_MONITOR:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.AIRSENSOR.getCapJsonStr();
			break;
		case THINGS_INFO.THINGS_AIRCON:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.AIRCON.getCapJsonStr();
			
			break;
		default:
			logger.error("Can not found device capability - " + modelId + " - " + things.getEndPoint());
			break;
		}
		Map<String, Object> capabilities = CommonUtil.jsonToMap(jsonString);
		logger.info("[" + modelId + "]-endpoint - " + things.getEndPoint() + "-capability="+ capabilities.toString());
		return capabilities;
	}
	
	public Map<String, Object> getAdditionalProp(String key, String value){
		Map<String, Object> additionalPropMap = new HashMap<String, Object>();
		additionalPropMap.put(key, value);
		return additionalPropMap;
	}
	
	
	public List<ThingsInfo> getChildThingsInfoList(int dong, int ho, int childCount, NodeList childList) {
		List<ThingsInfo> thingsList = new ArrayList<>();
		
		try {
			NodeUtil.asList(childList)
			.stream()
			.filter(cNode -> cNode != null)
			.forEach(cNode -> {
				try {
					Map<String, Object> deviceMap = new HashMap<>();
					String cNodeName = XmlUtil.getAttributeWithItem(cNode, "name");
					String cNodeDevNum = XmlUtil.getAttributeWithItem(cNode, "dev_num");
					String status = XmlUtil.getElemetTextValue(cNode);
					String endpointId = cNodeName+"_"+cNodeDevNum;
					
					String tmpModelId = endpointId;
					
					if(cNodeName.contains("light")) {
						String[] strSplit = status.split("/");
						// split 이 안됐다면 on off만 되는 예전 방식으로 전등 상태를 받았다는 뜻.
						if(strSplit.length > 1) {
							if(!strSplit[1].equals("null") && strSplit[2].equals("null")) {
								endpointId = "dimmer_"+endpointId;
							}
							else if(!strSplit[2].equals("null")) {
								endpointId = "colorDimmer_"+endpointId;
							}
							tmpModelId = cNodeName+"_"+cNodeDevNum;
						}
					}
					
		 			
					logger.debug("tmpModelId = " + tmpModelId + " || endpointID = " + endpointId);
					
		    		ThingsInfo thingsInfo = new ThingsInfo(endpointId, tmpModelId, Integer.toString(dong), Integer.toString(ho));
		    		
		    		thingsList.add(thingsInfo);
		    		
				}catch(Exception e) {
					e.printStackTrace();
				}
			});	
		}catch(Exception e) { 
			return null;
		}
		
		return thingsList;
	}
	
	/**
	 * Document에서 추출한 Device의 자식 노드들을 추출한다.
	 * @param childCount
	 * @param childList
	 * @return
	 */
	public List<Map<String, Object>> getChildDeviceInfo(int childCount, NodeList childList){
		List<Map<String, Object>> deviceList = new ArrayList<>();
		try {
			NodeUtil.asList(childList)
				.stream()
				.filter(cNode -> cNode != null)
				.forEach(cNode -> {
					try {
						Map<String, Object> deviceMap = new HashMap<>();
						String cNodeName = XmlUtil.getAttributeWithItem(cNode, "name");
						String cNodeDevNum = XmlUtil.getAttributeWithItem(cNode, "dev_num");
						String status = XmlUtil.getElemetTextValue(cNode);
						String[] strSplit = status.split("/");
						
						String endpointId = cNodeName+"_"+cNodeDevNum;
						
						if(cNodeName.contains("light")) {
							
							// split 이 안됐다면 on off만 되는 예전 방식으로 전등 상태를 받았다는 뜻.
							if(strSplit.length > 1) {
								if(!strSplit[1].equals("null") && strSplit[2].equals("null")) {
									endpointId = "dimmer_"+endpointId;
									cNodeName = "dimmer_" + cNodeName;
								}
								else if(!strSplit[2].equals("null")) {
									endpointId = "colorDimmer_"+endpointId;
									cNodeName = "colorDimmer_"+cNodeName;
								}
							}
						}
						
			 			String tmpModelId = cNodeName.equals("temper") || cNodeName.equals("aircon") ?endpointId:cNodeName;
						logger.debug("tmpModelId = " + tmpModelId + " || endpointID = " + endpointId + " || status = " + status);
			 			
						
						THINGS_INFO.DEVICE_DETAIL detailDev = THINGS_INFO.getDeviceInfo(tmpModelId, status);
						
						Map<String, Object> deviceInfo =  new HashMap<String, Object>();
						
						deviceInfo.put(KEY_INFO.BODY_KEY_ENDPOINT_ID, endpointId);
			    		deviceInfo.put(KEY_INFO.BODY_KEY_NAME, detailDev.getDisplayName()+cNodeDevNum);
			    		deviceInfo.put(KEY_INFO.BODY_KEY_DISPLAY_TYPE, detailDev.getDisplayType());
			    		
			    		deviceInfo.putAll(getDeviceCapabilities(endpointId, status));
			    		deviceInfo.put(KEY_INFO.BODY_KEY_STATE, setDeviceState(endpointId, status));
			    		deviceInfo.put(KEY_INFO.BODY_KEY_ADDITIONALPROP, getAdditionalProp(KEY_INFO.BODY_KEY_ROOMPRESETNAME,  detailDev.getRoomName()));
			    		
			    		deviceMap.put(endpointId, deviceInfo);
			    		
			    		if(!strSplit[0].equals("null"))
			    			deviceList.add(deviceMap);
					}catch(Exception e) {
						logger.error("[getChildDeviceInfo]-" + e.getMessage());
						e.printStackTrace();
					}
					
				});			
		}catch(Exception e) {
			return null;
		}
		return deviceList;
	}
	
	public List<Map<String, Object>> getChildDeviceInfo_bak(int childCount, NodeList childList){
		List<Map<String, Object>> deviceList = new ArrayList<>();
		
		for(int j=0; j < childCount; j++) {
			Map<String, Object> deviceMap = new HashMap<>();
			
			Node cNode = childList.item(j);
			if(cNode == null) {
				logger.error("cNode is null");
				return null;
			}
			
			String cNodeName = XmlUtil.getAttributeWithItem(cNode, "name");
			String cNodeDevNum = XmlUtil.getAttributeWithItem(cNode, "dev_num");
			String status = XmlUtil.getElemetTextValue(cNode);
			String endpointId = cNodeName+"_"+cNodeDevNum;
			
 			String tmpModelId = cNodeName.equals("temper")?endpointId:cNodeName;
			
			THINGS_INFO.DEVICE_DETAIL detailDev = THINGS_INFO.getDeviceInfo(tmpModelId, status);
			
			Map<String, Object> deviceInfo =  new HashMap<String, Object>();
			
			THINGS_INFO.THINGS things = THINGS_INFO.getThingsType(cNodeName);
			
			if(things.getEndPoint() == THINGS_INFO.THINGS_COLOR_LIGHT_DIMMER)
				endpointId = "colorDimming_"+endpointId;
			
			if(things.getEndPoint() == THINGS_INFO.THINGS_LIGHT_DIMMER) 
				endpointId = "dimming_"+endpointId;
			
			
			deviceInfo.put(KEY_INFO.BODY_KEY_ENDPOINT_ID, endpointId);
    		deviceInfo.put(KEY_INFO.BODY_KEY_NAME, detailDev.getDisplayName()+cNodeDevNum);
    		deviceInfo.put(KEY_INFO.BODY_KEY_DISPLAY_TYPE, detailDev.getDisplayType());
    		
    		deviceInfo.putAll(getDeviceCapabilities(endpointId, status));
    		deviceInfo.put(KEY_INFO.BODY_KEY_STATE, setDeviceState(endpointId, status));
    		
    		deviceMap.put(endpointId, deviceInfo);
    		deviceList.add(deviceMap);
		}
		
		return deviceList;
	}
	
	/**
	 * 디바이스의 상태 명령어를 디바이스 종류에 맞게 생성한다.
	 * 카카오에서 던져준 capability 값부터 유효성 검사를 한 뒤에 해당 capability와 맞는 속성인지도 확인해야 한다.
	 * @param modelId
	 * @param desired
	 * @return
	 */
	
	public static String setDeviceActionStr(int dong, int ho, String endPointId, String modelId, String capability, Map<String, Object> desired) {
		StringBuffer sb = new StringBuffer();
		
		THINGS_INFO.THINGS things = THINGS_INFO.getThingsType(endPointId);
		
		logger.debug("[setDeiceActionStr] - modelId = " + modelId + " || capability = " + capability + " || things endpoint = " + things.getEndPoint());
		logger.debug("ACTION STATUS --> " + desired.toString());
		try {
			int brightness = 1;
			int colorTemper = 1;
			switch(things.getEndPoint()) {
				
			case THINGS_INFO.THINGS_LIGHT:
				if(capability.equals("Power") && desired.containsKey("power")) {
					String tmpState = (String)desired.get("power");
					if(tmpState.equals("on") || tmpState.equals("off")) sb.append(tmpState);
					
					sb.append("/null/null");
				}
				
				break;
			case THINGS_INFO.THINGS_LIGHT_DIMMER:
				String [] arr_dimm_light_act = {"on", "null", "null"};
				
				if(desired.containsKey("power"))
					arr_dimm_light_act[0] = (String)desired.get("power");
					
				
				if(desired.containsKey("brightness")) {
//					brightness = ((int)desired.get("brightness"))/10 == 0 ? 1: ((int)desired.get("brightness"))/10;
					brightness = (int)desired.get("brightness");
					if(brightness < 1)
						arr_dimm_light_act[1] = Integer.toString(1);
					else if(brightness > 10)
						arr_dimm_light_act[1] = Integer.toString(10);
					else
						arr_dimm_light_act[1] = Integer.toString(brightness);
				}
				
				sb.append(arr_dimm_light_act[0]).append("/")
				  .append(arr_dimm_light_act[1]).append("/")
				  .append(arr_dimm_light_act[2]);
				break;
				
			case THINGS_INFO.THINGS_COLOR_LIGHT_DIMMER:
				String [] arr_color_dimm_light_act = {"on", "null", "null"};
				
				if(desired.containsKey("power"))
					arr_color_dimm_light_act[0] = (String)desired.get("power");
					
				
				if(desired.containsKey("brightness")) {
//					brightness = ((int)desired.get("brightness"))/10 == 0 ? 1: ((int)desired.get("brightness"))/10;
					brightness = (int)desired.get("brightness");
					if(brightness < 1)
						arr_color_dimm_light_act[1] = Integer.toString(1);
					else if(brightness > 10)
						arr_color_dimm_light_act[1] = Integer.toString(10);
					else
						arr_color_dimm_light_act[1] = Integer.toString(brightness);
				}
				
				if(desired.containsKey("colorTemperature")) {
//					colorTemper = ((int)desired.get("colorTemperature"))/1000 == 0 ? 1:((int)desired.get("colorTemperature"))/1000;
					int tmpColorTemp = (int)desired.get("colorTemperature");
					
					if(tmpColorTemp < 3000) tmpColorTemp = 3000;
					else if(tmpColorTemp > 5700) tmpColorTemp = 5700;
					
					colorTemper = (tmpColorTemp)/300 - 9;
					arr_color_dimm_light_act[2] = Integer.toString(colorTemper);
				}
				
				sb.append(arr_color_dimm_light_act[0]).append("/")
				  .append(arr_color_dimm_light_act[1]).append("/")
				  .append(arr_color_dimm_light_act[2]);
				break;
				
			case THINGS_INFO.THINGS_TEMPER:
				if(capability.equals("Power") && desired.containsKey("power")) {
					String tmpState = (String)desired.get("power");
					if(tmpState.equals("on") || tmpState.equals("off")) sb.append(tmpState);
					
					if(tmpState.equals("on") && desired.containsKey("targetSetpoint")) sb.append("/").append(desired.get("targetSetpoint"));
				}
					
				if(capability.equals("TemperatureSetpoint") && desired.containsKey("targetSetpoint")) {
					sb.append("on/").append(desired.get("targetSetpoint"));
				}
				
				
				break;
				
			case THINGS_INFO.THINGS_GAS:
				if(capability.equals("Lock") && desired.containsKey("locked")){
					if((boolean)desired.get("locked"))
						sb.append("close");
				}
				break;
				
			case THINGS_INFO.THINGS_VENTIL:
				String []arr_ventil_act = {"on", "null"};
				if(desired.containsKey("power")) {
					arr_ventil_act[0] = (String) desired.get("power");
				}
				if(desired.containsKey("fanSpeed")) {
					if("low".equals((String) desired.get("fanSpeed")))
						arr_ventil_act[1] = "low";
					else if("middle".equals((String) desired.get("fanSpeed")))
						arr_ventil_act[1] = "mid";
					else if("high".equals((String) desired.get("fanSpeed")))
						arr_ventil_act[1] = "high";
				}
				sb.append(arr_ventil_act[0]).append("/")
				  .append(arr_ventil_act[1]);
				break;
				
			case THINGS_INFO.THINGS_ELECPLUG: // 지원하지 않음
				if(capability.equals("Power") && desired.containsKey("power")) {
					String tmpState = (String)desired.get("power");
					if(tmpState.equals("on"))
						sb.append("on");
					else if(tmpState.equals("off")) sb.append("off");
				}
				break;
				
			case THINGS_INFO.THINGS_ELEVATOR:
				if(capability.equals("ElevatorCall") && desired.containsKey("callDirection")) {
					if("down".equalsIgnoreCase((String)desired.get("callDirection"))) {
						sb.append(desired.get("callDirection"));
					}
					
					logger.info("[Elevator Call]-" + sb.toString());
				}
				break;
			case THINGS_INFO.THINGS_AIRCON:
				//                         파워     설정온도    모드    풍속      풍향
				String[] arr_aircon_act = {"on", "null", "null", "null", "null"};
				
				if(desired.containsKey("power")) arr_aircon_act[0] = (String) desired.get("power");
				
				if(desired.containsKey("targetSetpoint")) {
					Map<String, Object> tempMap = getAirconEnableTemperature(dong, ho, endPointId);
					int minTemp = (int) tempMap.get("min");
					int maxTemp = (int) tempMap.get("max");
					double dobReqTemp = (double) desired.get("targetSetpoint");
					int reqTemp = (int) dobReqTemp;
					
//					int reqTemp = ((Double) desired.get("targetSetpoint")).intValue();
					if(reqTemp < minTemp) arr_aircon_act[1] = Integer.toString(minTemp);
					else if(reqTemp > maxTemp) arr_aircon_act[1] = Integer.toString(maxTemp);
					else arr_aircon_act[1] = Integer.toString(reqTemp);
				}
				
				if(desired.containsKey("mode")) {
					String modeVal = (String) desired.get("mode");
					
					if(modeVal.equals("auto"))
						arr_aircon_act[2] = "auto";
					else if(modeVal.equals("cool"))
						arr_aircon_act[2] = "cool";
					else if(modeVal.equals("heat"))
						arr_aircon_act[2] = "heat";
					else if(modeVal.equals("dehumidify"))
						arr_aircon_act[2] = "dehum";
					else if(modeVal.equals("fan_only"))
						arr_aircon_act[2] = "fan";
				}
				
				if(desired.containsKey("fanSpeed")) {
					String modeVal = (String) desired.get("fanSpeed");
					
					if(modeVal.equals("auto"))
						arr_aircon_act[3] = "auto";
					else if(modeVal.equals("low"))
						arr_aircon_act[3] = "low";
					else if(modeVal.equals("middle"))
						arr_aircon_act[3] = "mid";
					else if(modeVal.equals("high"))
						arr_aircon_act[3] = "high";
				}
				
				sb.append(arr_aircon_act[0]).append("/");
				sb.append(arr_aircon_act[1]).append("/");
				sb.append(arr_aircon_act[2]).append("/");
				sb.append(arr_aircon_act[3]).append("/null");
				
				logger.info("[aircon]-action str = " + sb.toString());
				break;
			}
		}catch(Exception e) {
			logger.error("[Action Str Error]-" + e.getMessage());
			e.printStackTrace();
		}
		
		return sb.toString();
	}
	
	private static int getAirQuaility(String type, int value) {
		switch(type) {
			case "co2":
				if(value < 1001)
					return 0;
				else if(value <= 1001 && value < 1501)
					return 1;
				else if(value <= 1501 && value < 3001)
					return 2;
				else return 3;
			case "pm10":
				if(value < 31)
					return 0;
				else if(value <= 31 && value < 81)
					return 1;
				else if(value <= 81 && value < 150)
					return 2;
				else return 3;
			case "pm2_5":
				if(value < 16)
					return 0;
				else if(value <= 16 && value < 36)
					return 1;
				else if(value <= 36 && value < 76)
					return 2;
				else return 3;
		}
		
		return 0;
	}
	/**
	 * 상태조회 이후 월패드에서 받은 상태 값을 디바이스의 속성에 맞게 파싱하고 
	 * Map에 <"속성명", "상태값"> 형태로 담아준다. 
	 * @param modelId
	 * @param status
	 * @return
	 */
	public static Map<String, Object> setDeviceState(String modelId, String status){
		Map<String,Object> state = new HashMap<String, Object>();
		
		THINGS_INFO.THINGS things = THINGS_INFO.getThingsType(modelId);
		String[] splitStr = status.split("\\/");
		
		if(splitStr[0].equals("null")) return state;
		
		int temp1, temp2 = 0;
		switch(things.getEndPoint()) {
		case THINGS_INFO.THINGS_ELECPLUG:
			state.put("power", splitStr[1]);
			
			break;
		case THINGS_INFO.THINGS_LIGHT:
		case THINGS_INFO.THINGS_LIGHT_DIMMER:
		case THINGS_INFO.THINGS_COLOR_LIGHT_DIMMER:
			
			state.put("power", splitStr[0]);
			
//			if(things.getEndPoint() == THINGS_INFO.THINGS_LIGHT) break; 
			if(splitStr.length == 1) break;
			
			if(splitStr[1] != null && !splitStr[1].equals("null")) {
				temp1 = Integer.parseInt(splitStr[1]);
				state.put("brightness", temp1);
			}
			
			if(splitStr[2] != null && !splitStr[2].equals("null")) {
				temp2 = ((Integer.parseInt(splitStr[2])+9)*300);
				state.put("colorMode", "colorTemperature");
				state.put("colorTemperature", temp2);
			}
			break;
		case THINGS_INFO.THINGS_TEMPER:
			Double targetTemp = Double.parseDouble(splitStr[1]);
			Double currTemp = Double.parseDouble(splitStr[2]);
			
			state.put("power", splitStr[0]);
			state.put("targetSetpoint", Double.valueOf(targetTemp.intValue()));
			state.put("currentTemperature", Double.valueOf(currTemp.intValue()));
			
			
			break;
		case THINGS_INFO.THINGS_GAS:
			if(status.equalsIgnoreCase("close"))
				state.put("locked", true);
			else
				state.put("locked", false);
			break;
			
		case THINGS_INFO.THINGS_VENTIL:
			state.put("power", splitStr[0]);
			if(splitStr[0].equalsIgnoreCase("on")) {
				if(splitStr.length == 1) state.put("fanSpeed", "low");
				else if(splitStr[1].equals("mid")) state.put("fanSpeed", "middle");
				else state.put("fanSpeed", splitStr[1]);
			}
			break;
		case THINGS_INFO.THINGS_ELEVATOR:
			state.put("callDirection", "down");
			break;
		case THINGS_INFO.THINGS_AIR_MONITOR:
			
			int val_co2 = (int) Double.parseDouble(splitStr[0]);
			int val_pm10 = (int) Double.parseDouble(splitStr[1]);
			int val_pm2_5 = (int) Double.parseDouble(splitStr[2]);
			
			int qual_co2 = getAirQuaility("co2", val_co2);
			int qual_pm10 = getAirQuaility("pm10", val_pm10);
			int qual_pm2_5 = getAirQuaility("pm2_5", val_pm2_5);
			
			String[] arr_quality = {"good", "moderate", "unhealthy", "very_unhealth"};
			int avg_qual = (qual_co2 + qual_pm10 + qual_pm2_5)/3;
			String qualityIndex = arr_quality[avg_qual];
			
			state.put("co2", val_co2);
			state.put("pm10", val_pm10);
			state.put("pm2_5", val_pm2_5);
			state.put("qualityIndex", qualityIndex);
			break;
		case THINGS_INFO.THINGS_AIRCON:
			logger.info("[에어컨 상태]-"+ status);
			
			String power = splitStr[0].equalsIgnoreCase("removeheat")?"off":splitStr[0];
			int set_temp = splitStr.length > 1 ? Integer.parseInt(splitStr[1]):0;
			int cur_temp = splitStr.length > 2 ?Integer.parseInt(splitStr[2]):0;
			String mode = splitStr.length > 3 ? splitStr[3]:null;
			String airFlow = splitStr.length > 4 ? splitStr[4]:null;
			
			state.put("power", power);
			state.put("targetSetpoint", set_temp);
			state.put("currentTemperature", cur_temp);
			
			if(mode != null) {
				if(mode.equals("auto")) state.put("mode", "auto");
				else if(mode.equals("cool")) state.put("mode", "cool");
				else if(mode.equals("heat")) state.put("mode", "heat");
				else if(mode.equals("dehum")) state.put("mode", "dehumidify");
				else if(mode.equals("fan")) state.put("mode", "fan_only");
			}
			
			if(airFlow != null) {
				if(airFlow.equals("auto")) state.put("fanSpeed", "auto");
				else if(airFlow.equals("low")) state.put("fanSpeed", "low");
				else if(airFlow.equals("mid")) state.put("fanSpeed", "middle");
				else if(airFlow.equals("high")) state.put("fanSpeed", "high");
			}
			break;
		}
		
		return state;
	}
}
