package com.icontrols.kakao.socket.wallpad;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.Socket;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.utils.BeanUtils;
import com.icontrols.kakao.common.utils.CommonUtil;
import com.icontrols.kakao.common.utils.JWTUtils2;
import com.icontrols.kakao.common.utils.XmlUtil;
import com.icontrols.kakao.config.AsyncConfig;
import com.icontrols.kakao.entity.DonghoInfo;
import com.icontrols.kakao.entity.ParcelInfo;
import com.icontrols.kakao.entity.UserInfo;
import com.icontrols.kakao.repo.DonghoInfoRepository;
import com.icontrols.kakao.repo.ParcelInfoRepository;
import com.icontrols.kakao.repo.UserInfoRepository;
import com.icontrols.kakao.service.AccessTokenService;

/**
 * 
 * @author 황의현 / 2019-03-13
 * 클라이언트로 데이터를 전송할 스레드 클래스
 * 
 * 월패드에서 전송한 이벤트 메세지를 받아서 처리를 하는 스레드 클래스
 *  
 */
public class ThreadServerHandler extends Thread {
	private static final Logger logger = LoggerFactory.getLogger(ThreadServerHandler.class);

	private Socket connectedClientSocket;					// 클라이언트와 연결된 소켓 객체
	
	private String strRcvMSG;								//  입력 받은 XML String 
	private Document doc;									// 
	
	static final int EVENTTIMELIMITE = 2000;
	static long lSocketConnectionPeriod = 0;
	static int TIMEOUT_iMAP_WAIT = 3000; // iMAP 타임아웃
	static int SOCKET_BUFFER_SIZE_READ = 50;
	static boolean newDanjiServerNoticeUpdate = false;

	
	// 생성자
	public ThreadServerHandler(Socket connectedClientSocket, boolean server) {
		
		// Client와 통신할 객체를 초기값으로 받아 할당합니다.
		this.connectedClientSocket = connectedClientSocket;
		this.newDanjiServerNoticeUpdate = server;
	}

	public void run() {
		InputStream InStm = null;
		InputStreamReader ISR = null;
		BufferedReader MsgReader = null;
		StringBuffer sbRcvMSG = null;
		boolean bRcvOK = false;
		try {
			// 클라이언트로 내용을 출력 할 객체 생성
			InStm = connectedClientSocket.getInputStream();									// Socket으로 연결된 클라이언트에게 Input Stream 을 얻는다.
			connectedClientSocket.setSoLinger(true, 3000);												// 소켓 옵션 설정 소켓이 닫힐 때 아직 전송되지 않은 데이터 처리에 대한 옵션 --> 3000초 의 linger time을 설정
			
			ISR = new InputStreamReader(InStm, "euc-kr");
			MsgReader = new BufferedReader(ISR);
			strRcvMSG = "";
			
			sbRcvMSG = new StringBuffer(); 												// Client에게 수신받은 iMap 메시지를 append한 뒤에 strRcvMSG에 할당한다.
			StringBuffer sbInBuffer = null;
			
//			String InBuffer;
			
			lSocketConnectionPeriod = System.currentTimeMillis() + TIMEOUT_iMAP_WAIT; // 타임아웃시간을 설정한다.

			while (System.currentTimeMillis() < lSocketConnectionPeriod) {
				if (MsgReader.ready()) {
					try {
						char[] ReadData = new char[SOCKET_BUFFER_SIZE_READ * 1024];
						sbInBuffer = new StringBuffer();
						while (MsgReader.ready()) {
							long charCnt = MsgReader.read(ReadData);
							sbInBuffer.append(new String(ReadData, 0, (int) charCnt));
						}

						if (sbInBuffer != null || sbInBuffer.toString().trim().length() > 0)
							sbRcvMSG.append(sbInBuffer.toString());
						
						if (0 < sbInBuffer.toString().indexOf("</imap>")) { // </imap> 으로 제대로 닫혔다면 클라이언트의 이벤트 메세지를 제대로 수신 했다는 말.
							bRcvOK = true;
							logger.debug("[iMAPSvrProc().run()] Received DONE!!");
							break;
						}
					} catch (Exception e) {
						logger.error("message : " + e);

						MsgReader.close();
						ISR.close();
						InStm.close();
						return;
					}
				} else {
					Thread.sleep(100);
				}
			}
		} catch (Exception e) {
			logger.error("[Exception] iMAPSvrProc.run() ===== 2 =====");
			logger.error("message : " + e);
			e.printStackTrace();

			try { connectedClientSocket.close(); } catch (IOException e1) { e1.printStackTrace(); }
			return;
		}finally {
			strRcvMSG = sbRcvMSG.toString();

			try { MsgReader.close(); } catch (IOException e) { e.printStackTrace(); }
			try { ISR.close(); } catch (IOException e) { e.printStackTrace(); }
			try { InStm.close(); } catch (IOException e) { e.printStackTrace(); }
		}
		
		try {
			if (!bRcvOK) {
				logger.error("[iMAPSvrProc().run()] iMAP message is unacceptable!! :\n\r" + strRcvMSG);
				connectedClientSocket.close();
				return;
			}
			
			// 응답을 제대로 받은 이후의 처리 .. 제대로 된 응다블 받지 못했다면 위에서 소켓 연결 해제 
			logger.debug("-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-");
			logger.debug("strRcvMSG :: > " + strRcvMSG);
			
			strRcvMSG = strRcvMSG.substring(strRcvMSG.indexOf("<")); // 이 부분에 대한 예시 데이터가 있으면 이해하기 좋을거 같다. 테스트 하면서 뽑아내고 주석으로 잘아보자.
			
			
			// XML문서를 읽어보자구요!																		
			// XML을 Document라는 객체로 인식
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();						// XML 문서를 읽기 위한 Factory 객체 생성
			DocumentBuilder builder = factory.newDocumentBuilder();										// Builder 객체 생성
			InputStream istream = new ByteArrayInputStream(strRcvMSG.getBytes("UTF-8"));
			doc = builder.parse(istream);

			// xpath 생성
			XPath xpath = XPathFactory.newInstance().newXPath();

			// id 가 c2 인 Node의 val attribute 값 가져오기
			Node service = (Node) xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
			String requestType = XmlUtil.getAttributeWithItem(service, "type");
			logger.debug("[ThreadServerHandler]-service type : " + requestType);
			
			String retXML = "";
			if (requestType.equalsIgnoreCase("notice")) {
				String requestCmd = XmlUtil.getAttributeWithItem(service, "name");
				
				logger.debug("[ThreadServerHandler]-requestCmd : " + requestCmd);
				if (requestCmd.equalsIgnoreCase("msg_device_event")) { // 상태변경이벤트인경우에 처리
					logger.debug("[ThreadServerHandler]-strRcvMSG : " + strRcvMSG);
					// 빈번하게 실행 될 가능성이 높다.
					DeviceEventMsgHandler emh = new DeviceEventMsgHandler(doc, requestCmd);
					emh.start();
				}else if (requestCmd.equalsIgnoreCase("msg_box_event")) { // 택배 이벤트인 경우에 처리
					logger.info("[ParcelEventMsgHandler]-[실행]");
					ParcelEventMsgHandler pmh = new ParcelEventMsgHandler(doc);
					pmh.start();
				}else if (requestCmd.equalsIgnoreCase("msg_dustinfo_event")) { // 공기질 이벤트인 경우에 처리
					logger.info("[DustEventMsgHandler]-[실행]");
					DustEventMsgHandler dmh = new DustEventMsgHandler(doc);
					dmh.start();
				}
			}
		} catch (Exception e) {
			logger.error("message : " + e);
		}finally {
			try { if(!connectedClientSocket.isClosed()) connectedClientSocket.close(); } catch (Exception E) { logger.error("message : " + E); }			
		}
	}

	// 클라이언트로 데이터를 전송할 스레드 클래스
	class DeviceEventMsgHandler extends Thread {
		// 멤버변수
		private Document doc;
		private String cmd;
		
		// 생성자
		public DeviceEventMsgHandler(Document doc, String cmd) {
			this.doc = doc;
		}

		public void run() {
			try {
				DonghoInfoRepository repo = (DonghoInfoRepository)BeanUtils.getBean("donghoInfoRepository");
				KakaoMsgHandler kakaoMsgHandler = (KakaoMsgHandler)BeanUtils.getBean("kakaoMsgHandler");
				
				// 데이터 뽑아서 이통사 연동 서버에 전달한다.
				String model_id, status;
				int dev_num;

				// xpath 생성
				XPath xpath = XPathFactory.newInstance().newXPath();

				// id 가 c2 인 Node의 val attribute 값 가져오기
				Node iMap = (Node) xpath.evaluate("//imap", doc, XPathConstants.NODE);
				logger.debug("[DeviceEventMsgHandler] - address : " + iMap.getAttributes().getNamedItem("address").getTextContent());
				logger.debug("[DeviceEventMsgHandler] - sender : "  + iMap.getAttributes().getNamedItem("sender").getTextContent());


				String dongho = XmlUtil.getAttributeWithItem(iMap, "sender");
				String searchDong = (dongho.indexOf("d") >= 0)?"d":"동";
				String searchHo = (dongho.indexOf("d") >= 0)?"h":"호";

				int idxdong = dongho.indexOf(searchDong);
				int idxho = dongho.indexOf(searchHo);

				String dong = dongho.substring(0, idxdong).trim();
				String ho = dongho.substring(idxdong + 1, idxho).trim();
				
				Node service = (Node) xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
				logger.debug(service.getAttributes().getNamedItem("type").getTextContent());

				model_id = (String) xpath.evaluate("//imap/service/model_id ", doc, XPathConstants.STRING);
				status = (String) xpath.evaluate("//imap/service/status", doc, XPathConstants.STRING);
				dev_num = status.equalsIgnoreCase("unsubscription")?1:Integer.parseInt((String) xpath.evaluate("//imap/service/dev_num", doc, XPathConstants.STRING));
				
				logger.debug("[DeviceEventMsgHandler] - model_id : " + model_id);
				logger.debug("[DeviceEventMsgHandler] - dev_num  : " + dev_num);
				logger.debug("[DeviceEventMsgHandler] - status   : " + status);

				logger.info("[DeviceEventMsgHandler] - [동:" + dong + "호:" + ho + "] - [" + model_id + "] - [" + dev_num + "] - ["+status+"]");
				String endpointId = model_id + "_" + dev_num;
				
				Map <String, Object> body = new HashMap<String, Object>();
				
//				String elkLogMessage = "[Device Status Event] ip: " + SERVERINFO.danziServerExIp + " dong: " + dong + " ho: " + ho +" model_id: " + model_id + " dev_num: " + dev_num + " status:  "+ status;
//				logger.info(elkLogMessage);
				
				AsyncConfig asyncConfig = (AsyncConfig)BeanUtils.getBean("asyncConfig");
				
					//dong ho 가  같은 유저 모두 조회
					
					List<DonghoInfo> dList = repo.findByDongAndHo(dong, ho);
					
					if(dList.size() == 0) {
						logger.info("[EVENT] - 해동 동 호 인원 없음 프로세스 종료");
						
						return;
					}
					
					//batchlight는 현재 카카오에서는 지원하지 않음
					if(endpointId.contains("batchlight")) return;
					
					KakaoEventHandler kakaoEventHandler = (KakaoEventHandler)BeanUtils.getBean("kakaoEventHandler");
					
					List<CompletableFuture<Map<String, Object>>> fList = new ArrayList<>();
					List<UserInfo> userSet = dList.get(0).getUserInfos();
					
					Map<String, Object> state = KakaoMsgHandler.setDeviceState(endpointId, status);
					
					userSet.stream()
						.filter(userInfo -> userInfo.getDeleteFlag().equals("N"))
						.forEach(userInfo -> {
						try {
							UserInfoRepository uRepo = (UserInfoRepository)BeanUtils.getBean("userInfoRepository");
							//사용자 access token이 유효한지 판단.
							int token_vali = JWTUtils2.validateToken(userInfo.getAccessToken());
							if(token_vali != KAKAO_RESULT.SUCCESS) {
								// 사용자 access token이 만료 됐으면 디비에 Update하자.
								userInfo.setDeleteFlag("Y");
								userInfo.setDeleteType("expiredToken");
								uRepo.save(userInfo);
							}else {
								Map<String, Object> sendMap = null;
								if(status.equalsIgnoreCase("unsubscription")) {
									// 구독해지 처리.
									userInfo.setDeleteFlag("Y");
									userInfo.setDeleteType("unsubscription");
									uRepo.save(userInfo);
									logger.info("[해지처리]-["+userInfo.getDonghoInfo().getDonghoId()+"]-[userID = " + userInfo.getId()+"]");
									// 삭제 이벤트 전송해야...
									sendMap = kakaoMsgHandler.setKDeleteIdsMsg(dong, ho, userInfo);
									logger.info("[UNSUBSCRIPTION] - [sendMap] - " + sendMap.toString());
								}else {
									if(state.isEmpty()) return;
									
									if(state.containsKey("colorTemperature") && state.containsKey("brightness")) {
										body.put("colorDimmer_"+endpointId, state);
									}else if(state.containsKey("brightness")) {
										body.put("dimmer_"+endpointId, state);	
									}else {
										body.put(endpointId, state);
									}
									
									logger.debug("body --> " + body.toString());
									
									sendMap = kakaoMsgHandler.setKakaoEventMsg(userInfo, body);
								}
								logger.debug("sendMap --> " + sendMap.toString());
								
								while(true) {
									if(asyncConfig.isEventTaskExecute()) {
										fList.add(kakaoEventHandler.sendEventAsync(userInfo, sendMap));
										break;
									}
									Thread.sleep(1000);
								}
							}
						}catch(Exception e) {
							logger.error("[ERROR - ASYNC EVENT SEND] - " + e.getMessage());
							e.printStackTrace();
						}
					});
					
					CompletableFuture.allOf(fList.toArray(new CompletableFuture[fList.size()])).join();
					// 이벤트 전송이 실패했는지 성공했는지..
					fList.stream().filter(future -> future != null).forEach(future -> { 
						try {
							Map<String, Object> eventResult = future.get();
							
							int code = (int)eventResult.get("result");
							String message = (String)eventResult.get("message");
							
							if(code == KAKAO_RESULT.SUCCESS) {
								logger.debug("[EVENT SUCCESS] - " + message);
							} else {
								logger.error("[EVENT ERROR] - [CODE-" + code + "]" + message);
								
							}							
						}catch(Exception e) {
							logger.error("[ERROR - future get] - " + e.getMessage());
							e.printStackTrace();
						}
					});
			} catch (Exception e) {
				logger.error("message : " + e);
				e.printStackTrace();
			}
		}
	}
	
	// 클라이언트로 데이터를 전송할 스레드 클래스
	class ParcelEventMsgHandler extends Thread {
		// 멤버변수
		private Document doc;
		
		// 생성자
		public ParcelEventMsgHandler(Document doc) {
			this.doc = doc;
		}

		public void run() {
			try {
				DonghoInfoRepository repo = (DonghoInfoRepository)BeanUtils.getBean("donghoInfoRepository");
				ParcelInfoRepository parcelRepo = (ParcelInfoRepository)BeanUtils.getBean("parcelInfoRepository");
				
				KakaoMsgHandler kakaoMsgHandler = (KakaoMsgHandler)BeanUtils.getBean("kakaoMsgHandler");
				Map <String, Object> body = new HashMap<String, Object>();
				
				// xpath 생성
				XPath xpath = XPathFactory.newInstance().newXPath();

				// id 가 c2 인 Node의 val attribute 값 가져오기
				Node iMap = (Node) xpath.evaluate("//imap", doc, XPathConstants.NODE);
				logger.debug("[ParcelEventMsgHandler] - address : " + iMap.getAttributes().getNamedItem("address").getTextContent());
				logger.debug("[ParcelEventMsgHandler] - sender : "  + iMap.getAttributes().getNamedItem("sender").getTextContent());


				String dongho = XmlUtil.getAttributeWithItem(iMap, "sender");
				String searchDong = (dongho.indexOf("d") >= 0)?"d":"동";
				String searchHo = (dongho.indexOf("d") >= 0)?"h":"호";

				int idxdong = dongho.indexOf(searchDong);
				int idxho = dongho.indexOf(searchHo);

				String dong = dongho.substring(0, idxdong).trim();
				String ho = dongho.substring(idxdong + 1, idxho).trim();
				
				Node service = (Node) xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
				logger.debug(service.getAttributes().getNamedItem("type").getTextContent());
				int quantity = (int)((double) xpath.evaluate("//imap/service/quantity", doc, XPathConstants.NUMBER));
				String moveInfo = (String) xpath.evaluate("//imap/service/move_info", doc, XPathConstants.STRING);
				String reportTime = (String) xpath.evaluate("//imap/service/report_time", doc, XPathConstants.STRING);
				//"2019-04-26T16:20:55Z" 형식을 "2019-07-09T13:25:31Z" 형식으로 변환해야함.
				String lastDeliveredTime = CommonUtil.convertTimeToKakaoTime(reportTime);
				logger.info("[ParcelEvent]-[dong="+dong+"]-[ho="+ ho +"]-[move_info="+ moveInfo + "]-[quantity="+ quantity + "]-[reportTime= "+reportTime+"]-[lastDeliveredTime= "+lastDeliveredTime+"]");
				
				List<ParcelInfo> list = parcelRepo.findByDonghoId(dong+"/"+ho);
				Map<String, Object> stateMap = new HashMap<>();
				
				if(list.size() == 1) {
					ParcelInfo parcelObj = list.get(0);
					
					parcelObj.setQuantity(quantity);
					parcelObj.setLstUptEvt(moveInfo);
					parcelObj.setLstUptTime(lastDeliveredTime);
					List<Object> parcelList = new ArrayList<>();
					
					for(int i=0; i < parcelObj.getQuantity(); i++) {
						Map<String, Object> _obj = new HashMap<>();
						_obj.put("name", "택배 보관함");
						_obj.put("lastDeliveredTime", parcelObj.getLstUptTime());
						parcelList.add(_obj);
					}
					
					stateMap.put("parcelLocations", parcelList);
					
					parcelRepo.save(parcelObj);
				}
				logger.info("[ParcelEventMsgHandler]-[stateMap]-"+ stateMap.toString());
				
				AsyncConfig asyncConfig = (AsyncConfig)BeanUtils.getBean("asyncConfig");
				
				//dong ho 가  같은 유저 모두 조회
				List<DonghoInfo> dList = repo.findByDongAndHo(dong, ho);
				
				if(dList.size() == 0) {
					logger.info("[EVENT] - 해동 동 호 인원 없음 프로세스 종료");
					return;
				}
				KakaoEventHandler kakaoEventHandler = (KakaoEventHandler)BeanUtils.getBean("kakaoEventHandler");
				
				List<CompletableFuture<Map<String, Object>>> fList = new ArrayList<>();
				List<UserInfo> userSet = dList.get(0).getUserInfos();
				
				userSet.stream()
					.filter(userInfo -> userInfo.getDeleteFlag().equals("N"))
					.forEach(userInfo -> {
					try {
						UserInfoRepository uRepo = (UserInfoRepository)BeanUtils.getBean("userInfoRepository");
						Map<String, Object> sendMap = null;
						if(stateMap.isEmpty()) return;
						
						body.put("parcel", stateMap);
						
						logger.debug("body --> " + body.toString());
						sendMap = kakaoMsgHandler.setKakaoEventMsg(userInfo, body);
						logger.debug("sendMap --> " + sendMap.toString());
						
						while(true) {
							if(asyncConfig.isEventTaskExecute()) {
								fList.add(kakaoEventHandler.sendEventAsync(userInfo, sendMap));
								break;
							}
							Thread.sleep(1000);
						}
					}catch(Exception e) {
						logger.error("[ERROR - ASYNC EVENT SEND] - " + e.getMessage());
						e.printStackTrace();
					}
				});
				
				CompletableFuture.allOf(fList.toArray(new CompletableFuture[fList.size()])).join();
				// 이벤트 전송이 실패했는지 성공했는지..
				fList.stream().filter(future -> future != null).forEach(future -> { 
					try {
						Map<String, Object> eventResult = future.get();
						
						int code = (int)eventResult.get("result");
						String message = (String)eventResult.get("message");
						
						if(code == KAKAO_RESULT.SUCCESS) {
							logger.debug("[EVENT SUCCESS] - " + message);
						} else {
							logger.error("[EVENT ERROR] - [CODE-" + code + "]" + message);
							
						}							
					}catch(Exception e) {
						logger.error("[ERROR - future get] - " + e.getMessage());
						e.printStackTrace();
					}
				});
			} catch (Exception e) {
				logger.error("message : " + e);
				e.printStackTrace();
			}
		}
	}
	
	// 클라이언트로 데이터를 전송할 스레드 클래스
		class DustEventMsgHandler extends Thread {
			// 멤버변수
			private Document doc;
			
			// 생성자
			public DustEventMsgHandler(Document doc) {
				this.doc = doc;
			}

			public void run() {
				try {
					DonghoInfoRepository repo = (DonghoInfoRepository)BeanUtils.getBean("donghoInfoRepository");
					
					KakaoMsgHandler kakaoMsgHandler = (KakaoMsgHandler)BeanUtils.getBean("kakaoMsgHandler");
					Map <String, Object> body = new HashMap<String, Object>();
					
					// xpath 생성
					XPath xpath = XPathFactory.newInstance().newXPath();

					// id 가 c2 인 Node의 val attribute 값 가져오기
					Node iMap = (Node) xpath.evaluate("//imap", doc, XPathConstants.NODE);
					logger.debug("[DustEventMsgHandler] - address : " + iMap.getAttributes().getNamedItem("address").getTextContent());
					logger.debug("[DustEventMsgHandler] - sender : "  + iMap.getAttributes().getNamedItem("sender").getTextContent());


					String dongho = XmlUtil.getAttributeWithItem(iMap, "sender");
					String searchDong = (dongho.indexOf("d") >= 0)?"d":"동";
					String searchHo = (dongho.indexOf("d") >= 0)?"h":"호";

					int idxdong = dongho.indexOf(searchDong);
					int idxho = dongho.indexOf(searchHo);

					String dong = dongho.substring(0, idxdong).trim();
					String ho = dongho.substring(idxdong + 1, idxho).trim();
					
					
					String model_id = (String) xpath.evaluate("//imap/service/model_id ", doc, XPathConstants.STRING);
					String dev_num = (String) xpath.evaluate("//imap/service/dev_num", doc, XPathConstants.STRING);
					String endpointId = model_id + "_" + dev_num;
					
					Node service = (Node) xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
					logger.debug(service.getAttributes().getNamedItem("type").getTextContent());
					
					Map<String, Object> stateMap = new HashMap<>();
					
					int val_co2 = (int)(double) xpath.evaluate("//imap/service/co2", doc, XPathConstants.NUMBER);
					int val_pm10 = (int)(double) xpath.evaluate("//imap/service/fine_dust", doc, XPathConstants.NUMBER);
					int val_pm2_5 = (int)(double) xpath.evaluate("//imap/service/ultrafine_dust", doc, XPathConstants.NUMBER);
					
					int qual_co2 = CommonUtil.getAirQuaility("co2", val_co2);
					int qual_pm10 = CommonUtil.getAirQuaility("pm10", val_pm10);
					int qual_pm2_5 = CommonUtil.getAirQuaility("pm2_5", val_pm2_5);
					
					String[] arr_quality = {"good", "moderate", "unhealthy", "very_unhealth"};
					int avg_qual = (qual_co2 + qual_pm10 + qual_pm2_5)/3;
					String qualityIndex = arr_quality[avg_qual];
					
					stateMap.put("co2", val_co2);
					stateMap.put("pm10", val_pm10);
					stateMap.put("pm2_5", val_pm2_5);
					stateMap.put("qualityIndex", qualityIndex);
					
					logger.info("[DustEventMsgHandler]-[stateMap]-"+ stateMap.toString());
					AsyncConfig asyncConfig = (AsyncConfig)BeanUtils.getBean("asyncConfig");
					
					//dong ho 가  같은 유저 모두 조회
					List<DonghoInfo> dList = repo.findByDongAndHo(dong, ho);
					
					if(dList.size() == 0) {
						logger.info("[EVENT] - 해동 동 호 인원 없음 프로세스 종료");
						return;
					}
					
					KakaoEventHandler kakaoEventHandler = (KakaoEventHandler)BeanUtils.getBean("kakaoEventHandler");
					
					List<CompletableFuture<Map<String, Object>>> fList = new ArrayList<>();
					List<UserInfo> userSet = dList.get(0).getUserInfos();
					
					userSet.stream()
						.filter(userInfo -> userInfo.getDeleteFlag().equals("N"))
						.forEach(userInfo -> {
							try {
								UserInfoRepository uRepo = (UserInfoRepository)BeanUtils.getBean("userInfoRepository");
								Map<String, Object> sendMap = null;
								if(stateMap.isEmpty()) return;
								
								body.put(endpointId, stateMap);
								
								logger.debug("body --> " + body.toString());
								sendMap = kakaoMsgHandler.setKakaoEventMsg(userInfo, body);
								logger.debug("sendMap --> " + sendMap.toString());
								
								while(true) {
									if(asyncConfig.isEventTaskExecute()) {
										fList.add(kakaoEventHandler.sendEventAsync(userInfo, sendMap));
										break;
									}
									Thread.sleep(1000);
								}
							}catch(Exception e) {
								logger.error("[ERROR - ASYNC EVENT SEND] - " + e.getMessage());
								e.printStackTrace();
							}
						});
					
					CompletableFuture.allOf(fList.toArray(new CompletableFuture[fList.size()])).join();
					// 이벤트 전송이 실패했는지 성공했는지..
					fList.stream().filter(future -> future != null).forEach(future -> { 
						try {
							Map<String, Object> eventResult = future.get();
							
							int code = (int)eventResult.get("result");
							String message = (String)eventResult.get("message");
							
							if(code == KAKAO_RESULT.SUCCESS) {
								logger.debug("[EVENT SUCCESS] - " + message);
							} else {
								logger.error("[EVENT ERROR] - [CODE-" + code + "]" + message);
								
							}							
						}catch(Exception e) {
							logger.error("[ERROR - future get] - " + e.getMessage());
							e.printStackTrace();
						}
					});
				} catch (Exception e) {
					logger.error("message : " + e);
					e.printStackTrace();
				}
			}
		}
}
